@@ -231,29 +231,13 @@ class DhcpAgent(manager.Manager):
         if not network.admin_state_up:
             return
 
-        enable_metadata = self.dhcp_driver_cls.should_enable_metadata(
-                self.conf, network)
-        dhcp_network_enabled = False
-
         for subnet in network.subnets:
             if subnet.enable_dhcp:
                 if self.call_driver('enable', network):
-                    dhcp_network_enabled = True
+                    self.update_isolated_metadata_proxy(network)
                     self.cache.put(network)
                 break
 
-        if enable_metadata and dhcp_network_enabled:
-            for subnet in network.subnets:
-                if subnet.ip_version == 4 and subnet.enable_dhcp:
-                    self.enable_isolated_metadata_proxy(network)
-                    break
-        elif (not self.conf.force_metadata and
-              not self.conf.enable_isolated_metadata):
-            # In the case that the dhcp agent ran with metadata enabled,
-            # and dhcp agent now starts with metadata disabled, check and
-            # delete any metadata_proxy.
-            self.disable_isolated_metadata_proxy(network)
-
     def disable_dhcp_helper(self, network_id):
         """Disable DHCP for a network known to the agent."""
         network = self.cache.get_network_by_id(network_id)
@@ -280,17 +264,21 @@ class DhcpAgent(manager.Manager):
         if not network:
             return
 
-        old_cidrs = set(s.cidr for s in old_network.subnets if s.enable_dhcp)
-        new_cidrs = set(s.cidr for s in network.subnets if s.enable_dhcp)
-
-        if new_cidrs and old_cidrs == new_cidrs:
+        if not any(s for s in network.subnets if s.enable_dhcp):
+            self.disable_dhcp_helper(network.id)
+            return
+        # NOTE(kevinbenton): we don't exclude dhcp disabled subnets because
+        # they still change the indexes used for tags
+        old_cidrs = [s.cidr for s in network.subnets]
+        new_cidrs = [s.cidr for s in old_network.subnets]
+        if old_cidrs == new_cidrs:
             self.call_driver('reload_allocations', network)
             self.cache.put(network)
-        elif new_cidrs:
-            if self.call_driver('restart', network):
-                self.cache.put(network)
-        else:
-            self.disable_dhcp_helper(network.id)
+        elif self.call_driver('restart', network):
+            self.cache.put(network)
+
+        # Update the metadata proxy after the dhcp driver has been updated
+        self.update_isolated_metadata_proxy(network)
 
     @utils.synchronized('dhcp-agent')
     def network_create_end(self, context, payload):
@@ -333,6 +321,9 @@ class DhcpAgent(manager.Manager):
     def port_update_end(self, context, payload):
         """Handle the port.update.end notification event."""
         updated_port = dhcp.DictModel(payload['port'])
+        if self.cache.is_port_message_stale(payload['port']):
+            LOG.debug("Discarding stale port update: %s", updated_port)
+            return
         network = self.cache.get_network_by_id(updated_port.network_id)
         if network:
             LOG.info(_LI("Trigger reload_allocations for port %s"),
@@ -360,11 +351,26 @@ class DhcpAgent(manager.Manager):
     def port_delete_end(self, context, payload):
         """Handle the port.delete.end notification event."""
         port = self.cache.get_port_by_id(payload['port_id'])
+        self.cache.deleted_ports.add(payload['port_id'])
         if port:
             network = self.cache.get_network_by_id(port.network_id)
             self.cache.remove_port(port)
             self.call_driver('reload_allocations', network)
 
+    def update_isolated_metadata_proxy(self, network):
+        """Spawn or kill metadata proxy.
+
+        According to return from driver class, spawn or kill the metadata
+        proxy process. Spawn an existing metadata proxy or kill a nonexistent
+        metadata proxy will just silently return.
+        """
+        should_enable_metadata = self.dhcp_driver_cls.should_enable_metadata(
+            self.conf, network)
+        if should_enable_metadata:
+            self.enable_isolated_metadata_proxy(network)
+        else:
+            self.disable_isolated_metadata_proxy(network)
+
     def enable_isolated_metadata_proxy(self, network):
 
         # The proxy might work for either a single network
@@ -477,6 +483,15 @@ class NetworkCache(object):
         self.cache = {}
         self.subnet_lookup = {}
         self.port_lookup = {}
+        self.deleted_ports = set()
+
+    def is_port_message_stale(self, payload):
+        orig = self.get_port_by_id(payload['id']) or {}
+        if orig.get('revision_number', 0) > payload.get('revision_number', 0):
+            return True
+        if payload['id'] in self.deleted_ports:
+            return True
+        return False
 
     def get_network_ids(self):
         return self.cache.keys()
