@@ -521,10 +521,8 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
         if fip_agent_port:
             LOG.debug("FloatingIP agent gateway port received from the "
                 "plugin: %s", fip_agent_port)
-        is_first = False
         if floating_ips:
-            is_first = self.fip_ns.subscribe(ex_gw_port['network_id'])
-            if is_first and not fip_agent_port:
+            if not fip_agent_port:
                 LOG.debug("No FloatingIP agent gateway port possibly due to "
                           "late binding of the private port to the host, "
                           "requesting agent gateway port for 'network-id' :"
@@ -539,10 +537,7 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
                 if 'subnets' not in fip_agent_port:
                     LOG.error(_LE('Missing subnet/agent_gateway_port'))
                 else:
-                    if is_first:
-                        self.fip_ns.create_gateway_port(fip_agent_port)
-                    else:
-                        self.fip_ns.update_gateway_port(fip_agent_port)
+                    self.fip_ns.create_or_update_gateway_port(fip_agent_port)
 
             if (self.fip_ns.agent_gateway_port and
                 (self.dist_fip_count == 0)):
