@@ -16,6 +16,7 @@ import os
 import sys
 
 import httplib2
+import netaddr
 from oslo_config import cfg
 from oslo_log import log as logging
 import requests
@@ -23,6 +24,7 @@ import requests
 from neutron._i18n import _, _LE
 from neutron.agent.l3 import ha
 from neutron.agent.linux import daemon
+from neutron.agent.linux import ip_lib
 from neutron.agent.linux import ip_monitor
 from neutron.agent.linux import utils as agent_utils
 from neutron.common import config
@@ -31,6 +33,10 @@ from neutron.common import config
 LOG = logging.getLogger(__name__)
 
 
+class L3HAConfig(object):
+    send_arp_for_ha = 3
+
+
 class KeepalivedUnixDomainConnection(agent_utils.UnixDomainHTTPConnection):
     def __init__(self, *args, **kwargs):
         # Old style super initialization is required!
@@ -49,20 +55,21 @@ class MonitorDaemon(daemon.Daemon):
         self.conf_dir = conf_dir
         self.interface = interface
         self.cidr = cidr
+        self.monitor = None
         super(MonitorDaemon, self).__init__(pidfile, uuid=router_id,
                                             user=user, group=group)
 
     def run(self, run_as_root=False):
-        monitor = ip_monitor.IPMonitor(namespace=self.namespace,
-                                       run_as_root=run_as_root)
-        monitor.start()
+        self.monitor = ip_monitor.IPMonitor(namespace=self.namespace,
+                                            run_as_root=run_as_root)
+        self.monitor.start()
         # Only drop privileges if the process is currently running as root
         # (The run_as_root variable name here is unfortunate - It means to
         # use a root helper when the running process is NOT already running
         # as root
         if not run_as_root:
             super(MonitorDaemon, self).run()
-        for iterable in monitor:
+        for iterable in self.monitor:
             self.parse_and_handle_event(iterable)
 
     def parse_and_handle_event(self, iterable):
@@ -72,6 +79,15 @@ class MonitorDaemon(daemon.Daemon):
                 new_state = 'master' if event.added else 'backup'
                 self.write_state_change(new_state)
                 self.notify_agent(new_state)
+            elif event.interface != self.interface and event.added:
+                # Send GARPs for all new router interfaces.
+                # REVISIT(jlibosva): keepalived versions 1.2.19 and below
+                # contain bug where gratuitous ARPs are not sent on receiving
+                # SIGHUP signal. This is a workaround to this bug. keepalived
+                # has this issue fixed since 1.2.20 but the version is not
+                # packaged in some distributions (RHEL/CentOS/Ubuntu Xenial).
+                # Remove this code once new keepalived versions are available.
+                self.send_garp(event)
         except Exception:
             LOG.exception(_LE(
                 'Failed to process or handle event for line %s'), iterable)
@@ -96,6 +112,16 @@ class MonitorDaemon(daemon.Daemon):
 
         LOG.debug('Notified agent router %s, state %s', self.router_id, state)
 
+    def send_garp(self, event):
+        """Send gratuitous ARP for given event."""
+        ip_lib.send_ip_addr_adv_notif(
+            self.namespace,
+            event.interface,
+            str(netaddr.IPNetwork(event.cidr).ip),
+            L3HAConfig,
+            log_exception=False
+        )
+
 
 def register_opts(conf):
     conf.register_cli_opt(
