@@ -369,6 +369,11 @@ class Dnsmasq(DhcpLocalProcess):
                                ('set:', self._TAG_PREFIX % i,
                                 cidr.network, mode, lease))
                 else:
+                    if cidr.prefixlen < 64:
+                        LOG.debug('Ignoring subnet %(subnet)s, CIDR has '
+                                  'prefix length < 64: %(cidr)s',
+                                  {'subnet': subnet.id, 'cidr': cidr})
+                        continue
                     cmd.append('--dhcp-range=%s%s,%s,%s,%d,%s' %
                                ('set:', self._TAG_PREFIX % i,
                                 cidr.network, mode,
@@ -678,8 +683,7 @@ class Dnsmasq(DhcpLocalProcess):
                           (port.mac_address, name, ip_address))
 
         common_utils.replace_file(filename, buf.getvalue())
-        LOG.debug('Done building host file %s with contents:\n%s', filename,
-                  buf.getvalue())
+        LOG.debug('Done building host file %s', filename)
         return filename
 
     def _get_client_id(self, port):
@@ -969,7 +973,9 @@ class Dnsmasq(DhcpLocalProcess):
         with 3rd party backends.
         """
         if conf.force_metadata:
-            return True
+            # Only ipv4 subnet, with dhcp enabled, will use metadata proxy.
+            return any(s for s in network.subnets
+                       if s.ip_version == 4 and s.enable_dhcp)
 
         if conf.enable_metadata_network and conf.enable_isolated_metadata:
             # check if the network has a metadata subnet
@@ -982,7 +988,10 @@ class Dnsmasq(DhcpLocalProcess):
             return False
 
         isolated_subnets = cls.get_isolated_subnets(network)
-        return any(isolated_subnets[subnet.id] for subnet in network.subnets)
+        # Only ipv4 isolated subnet, which has dhcp enabled, will use
+        # metadata proxy.
+        return any(isolated_subnets[s.id] for s in network.subnets
+                   if s.ip_version == 4 and s.enable_dhcp)
 
 
 class DeviceManager(object):
@@ -1259,7 +1268,7 @@ class DeviceManager(object):
                     net = netaddr.IPNetwork(subnet.cidr)
                     ip_cidrs.append('%s/%s' % (gateway, net.prefixlen))
 
-        if self.conf.enable_isolated_metadata:
+        if self.conf.force_metadata or self.conf.enable_isolated_metadata:
             ip_cidrs.append(METADATA_DEFAULT_CIDR)
 
         self.driver.init_l3(interface_name, ip_cidrs,
@@ -1293,7 +1302,7 @@ class DeviceManager(object):
         """Ensure DHCP reply packets always have correct UDP checksums."""
         iptables_mgr = iptables_manager.IptablesManager(use_ipv6=False,
                                                         namespace=namespace)
-        ipv4_rule = ('-p udp --dport %d -j CHECKSUM --checksum-fill'
+        ipv4_rule = ('-p udp -m udp --dport %d -j CHECKSUM --checksum-fill'
                      % constants.DHCP_RESPONSE_PORT)
         iptables_mgr.ipv4['mangle'].add_rule('POSTROUTING', ipv4_rule)
         iptables_mgr.apply()
