@@ -16,9 +16,22 @@
 from neutron.common import constants
 
 OF_STATE_NOT_TRACKED = "-trk"
-OF_STATE_ESTABLISHED = "+trk+est-rel-inv"
-OF_STATE_RELATED = "+trk+rel-est-inv"
+OF_STATE_TRACKED = "+trk"
+OF_STATE_NEW_NOT_ESTABLISHED = "+new-est"
+OF_STATE_NOT_ESTABLISHED = "-est"
+OF_STATE_ESTABLISHED = "+est"
+OF_STATE_ESTABLISHED_NOT_REPLY = "+est-rel-rpl"
+OF_STATE_ESTABLISHED_REPLY = "+est-rel+rpl"
+OF_STATE_RELATED = "-new-est+rel-inv"
 OF_STATE_INVALID = "+trk+inv"
+OF_STATE_NEW = "+new"
+OF_STATE_NOT_REPLY_NOT_NEW = "-new-rpl"
+
+CT_MARK_NORMAL = '0x0'
+CT_MARK_INVALID = '0x1'
+
+REG_PORT = 5
+REG_NET = 6
 
 protocol_to_nw_proto = {
         constants.PROTO_NAME_ICMP: constants.PROTO_NUM_ICMP,
