@@ -91,6 +91,9 @@ class Connection(object):
         while True:
             self.idl.wait(self.poller)
             self.poller.fd_wait(self.txns.alert_fileno, poller.POLLIN)
+            #TODO(jlibosva): Remove next line once losing connection to ovsdb
+            #                is solved.
+            self.poller.timer_wait(self.timeout * 1000)
             self.poller.block()
             self.idl.run()
             txn = self.txns.get_nowait()
