@@ -103,12 +103,7 @@ class SecurityGroupAgentRpc(object):
         try:
             self.firewall = firewall_class(
                 integration_bridge=integration_bridge)
-        except TypeError as e:
-            LOG.warning(_LW("Firewall driver %(fw_driver)s doesn't accept "
-                            "integration_bridge parameter in __init__(): "
-                            "%(err)s"),
-                        {'fw_driver': firewall_driver,
-                         'err': e})
+        except TypeError:
             self.firewall = firewall_class()
         # The following flag will be set to true if port filter must not be
         # applied as soon as a rule or membership notification is received
@@ -302,7 +297,7 @@ class SecurityGroupAgentRpc(object):
             LOG.debug("Refreshing firewall for all filtered devices")
             self.refresh_firewall()
         else:
-            if self.use_enhanced_rpc:
+            if self.use_enhanced_rpc and updated_devices:
                 self.firewall.security_group_updated('sg_member', [],
                                                      updated_devices)
             # If a device is both in new and updated devices
