@@ -10,6 +10,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from neutron._i18n import _
 from neutron.common import exceptions
 
 
@@ -23,3 +24,9 @@ class CallbackNotFound(exceptions.NeutronException):
 
 class CallbacksMaxLimitReached(exceptions.NeutronException):
     message = _("Cannot add multiple callbacks for %(resource_type)s")
+
+
+class NoAgentDbMixinImplemented(exceptions.NeutronException):
+    message = _("RPC callbacks mechanism needs the implementation of "
+                "AgentDbMixin in the plugin, as so far it's only designed "
+                "to work with agents")
