@@ -10,20 +10,24 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo_config import cfg
+
+from neutron import server
 from neutron.server import rpc_eventlet
 from neutron.server import wsgi_eventlet
 from neutron.server import wsgi_pecan
 
 
-def main_wsgi_eventlet():
-    wsgi_eventlet.main()
+def main():
+    server.boot_server(_main_neutron_server)
 
 
-# Eventlet patching is not required for Pecan, but some plugins still spawn
-# eventlet threads
-def main_wsgi_pecan():
-    wsgi_pecan.main()
+def _main_neutron_server():
+    if cfg.CONF.web_framework == 'legacy':
+        wsgi_eventlet.eventlet_wsgi_server()
+    else:
+        wsgi_pecan.pecan_wsgi_server()
 
 
 def main_rpc_eventlet():
-    rpc_eventlet.main()
+    server.boot_server(rpc_eventlet.eventlet_rpc_server)
