@@ -186,7 +186,10 @@ core_opts = [
                       'this value without modification. For overlay networks '
                       'such as VXLAN, neutron automatically subtracts the '
                       'overlay protocol overhead from this value. Defaults '
-                      'to 1500, the standard value for Ethernet.'))
+                      'to 1500, the standard value for Ethernet. If using the '
+                      'ML2 plug-in with overlay/tunnel networks, also '
+                      'configure the ml2 path_mtu option with the same value '
+                      'as the global_physnet_mtu option.'))
 ]
 
 core_cli_opts = [
