@@ -21,6 +21,19 @@ from neutron._i18n import _
 from neutron.common import _deprecate
 
 
+class MultipleExceptions(Exception):
+    """Container for multiple exceptions encountered.
+
+    The API layer of Neutron will automatically unpack, translate,
+    filter, and combine the inner exceptions in any exception derived
+    from this class.
+    """
+
+    def __init__(self, exceptions, *args, **kwargs):
+        super(MultipleExceptions, self).__init__(*args, **kwargs)
+        self.inner_exceptions = exceptions
+
+
 class SubnetPoolNotFound(e.NotFound):
     message = _("Subnet pool %(subnetpool_id)s could not be found.")
 
