@@ -21,8 +21,8 @@ import os
 import netaddr
 from oslo_log import log
 
+from neutron._i18n import _, _LI
 from neutron.common import constants
-from neutron.i18n import _LI
 
 
 LOG = log.getLogger(__name__)
