@@ -31,8 +31,12 @@ from sqlalchemy import sql
 from neutron._i18n import _, _LE, _LI, _LW
 from neutron.api.rpc.callbacks import version_manager
 from neutron.api.v2 import attributes
+from neutron.callbacks import events
+from neutron.callbacks import registry
+from neutron.callbacks import resources
 from neutron.common import constants
 from neutron import context
+from neutron.db import api as db_api
 from neutron.db import model_base
 from neutron.extensions import agent as ext_agent
 from neutron.extensions import availability_zone as az_ext
@@ -173,11 +177,6 @@ class AgentAvailabilityZoneMixin(az_ext.AvailabilityZonePluginBase):
 class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
     """Mixin class to add agent extension to db_base_plugin_v2."""
 
-    def __init__(self, *args, **kwargs):
-        version_manager.set_consumer_versions_callback(
-            self._get_agents_resource_versions)
-        super(AgentDbMixin, self).__init__(*args, **kwargs)
-
     def _get_agent(self, context, id):
         try:
             agent = self._get_by_id(context, Agent, id)
@@ -216,15 +215,18 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
     def get_configuration_dict(self, agent_db):
         return self._get_dict(agent_db, 'configurations')
 
-    def _get_dict(self, agent_db, dict_name):
+    def _get_dict(self, agent_db, dict_name, ignore_missing=False):
+        json_value = None
         try:
-            conf = jsonutils.loads(getattr(agent_db, dict_name))
+            json_value = getattr(agent_db, dict_name)
+            conf = jsonutils.loads(json_value)
         except Exception:
-            msg = _LW('Dictionary %(dict_name)s for agent %(agent_type)s on '
-                      'host %(host)s is invalid.')
-            LOG.warning(msg, {'dict_name': dict_name,
-                              'agent_type': agent_db.agent_type,
-                              'host': agent_db.host})
+            if json_value or not ignore_missing:
+                msg = _LW('Dictionary %(dict_name)s for agent %(agent_type)s '
+                          'on host %(host)s is invalid.')
+                LOG.warning(msg, {'dict_name': dict_name,
+                                  'agent_type': agent_db.agent_type,
+                                  'host': agent_db.host})
             conf = {}
         return conf
 
@@ -245,13 +247,16 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
                    if k not in ['alive', 'configurations'])
         res['alive'] = not self.is_agent_down(res['heartbeat_timestamp'])
         res['configurations'] = self._get_dict(agent, 'configurations')
-        res['resource_versions'] = self._get_dict(agent, 'resource_versions')
+        res['resource_versions'] = self._get_dict(agent, 'resource_versions',
+                                                  ignore_missing=True)
         res['availability_zone'] = agent['availability_zone']
         return self._fields(res, fields)
 
     def delete_agent(self, context, id):
+        agent = self._get_agent(context, id)
+        registry.notify(resources.AGENT, events.BEFORE_DELETE, self,
+                        context=context, agent=agent)
         with context.session.begin(subtransactions=True):
-            agent = self._get_agent(context, id)
             context.session.delete(agent)
 
     def update_agent(self, context, id, agent):
@@ -336,8 +341,10 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
                 res['availability_zone'] = agent_state['availability_zone']
             configurations_dict = agent_state.get('configurations', {})
             res['configurations'] = jsonutils.dumps(configurations_dict)
-            resource_versions_dict = agent_state.get('resource_versions', {})
-            res['resource_versions'] = jsonutils.dumps(resource_versions_dict)
+            resource_versions_dict = agent_state.get('resource_versions')
+            if resource_versions_dict:
+                res['resource_versions'] = jsonutils.dumps(
+                    resource_versions_dict)
             res['load'] = self._get_agent_load(agent_state)
             current_time = timeutils.utcnow()
             try:
@@ -345,6 +352,13 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
                     context, agent_state['agent_type'], agent_state['host'])
                 if not agent_db.is_active:
                     status = constants.AGENT_REVIVED
+                    if 'resource_versions' not in agent_state:
+                        # updating agent_state with resource_versions taken
+                        # from db so that
+                        # _update_local_agent_resource_versions() will call
+                        # version_manager and bring it up to date
+                        agent_state['resource_versions'] = self._get_dict(
+                            agent_db, 'resource_versions', ignore_missing=True)
                 res['heartbeat_timestamp'] = current_time
                 if agent_state.get('start_flag'):
                     res['started_at'] = current_time
@@ -389,14 +403,15 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
                                     filters={'admin_state_up': [True]})
         return filter(self.is_agent_considered_for_versions, up_agents)
 
-    def _get_agents_resource_versions(self, tracker):
+    def get_agents_resource_versions(self, tracker):
         """Get the known agent resource versions and update the tracker.
 
-        Receives a version_manager.ResourceConsumerTracker instance and it's
-        expected to look up in to the database and update every agent resource
-        versions.
+        This function looks up into the database and updates every agent
+        resource versions.
         This method is called from version_manager when the cached information
         has passed TTL.
+
+        :param tracker: receives a version_manager.ResourceConsumerTracker
         """
         for agent in self._get_agents_considered_for_versions():
             resource_versions = agent.get('resource_versions', {})
@@ -434,6 +449,7 @@ class AgentExtRpcCallback(object):
         # Initialize RPC api directed to other neutron-servers
         self.server_versions_rpc = resources_rpc.ResourcesPushToServersRpcApi()
 
+    @db_api.retry_db_errors
     def report_state(self, context, **kwargs):
         """Report state from agent to server.
 
@@ -458,7 +474,10 @@ class AgentExtRpcCallback(object):
         return agent_status
 
     def _update_local_agent_resource_versions(self, context, agent_state):
-        resource_versions_dict = agent_state.get('resource_versions', {})
+        resource_versions_dict = agent_state.get('resource_versions')
+        if not resource_versions_dict:
+            return
+
         version_manager.update_versions(
             version_manager.AgentConsumer(agent_type=agent_state['agent_type'],
                                           host=agent_state['host']),
