@@ -266,7 +266,7 @@ class BgpDbMixin(common_db.CommonDbMixin):
         bp = bgp_peer[bgp_ext.BGP_PEER_BODY_KEY_NAME]
         with context.session.begin(subtransactions=True):
             bgp_peer_db = self._get_bgp_peer(context, bgp_peer_id)
-            if ((bp['password'] is not None) and
+            if ((bp.get('password') is not None) and
                 (bgp_peer_db['auth_type'] == 'none')):
                 raise bgp_ext.BgpPeerNotAuthenticated(bgp_peer_id=bgp_peer_id)
             bgp_peer_db.update(bp)
@@ -434,8 +434,9 @@ class BgpDbMixin(common_db.CommonDbMixin):
     def _get_bgp_speaker_network_binding(self, context,
                                          bgp_speaker_id, network_id):
         query = self._model_query(context, BgpSpeakerNetworkBinding)
-        return query.filter(bgp_speaker_id == bgp_speaker_id,
-                            network_id == network_id).one()
+        return query.filter(
+                    BgpSpeakerNetworkBinding.bgp_speaker_id == bgp_speaker_id,
+                    BgpSpeakerNetworkBinding.network_id == network_id).one()
 
     def _make_bgp_peer_dict(self, bgp_peer, fields=None):
         attrs = ['tenant_id', 'id', 'name', 'peer_ip', 'remote_as',
@@ -696,10 +697,15 @@ class BgpDbMixin(common_db.CommonDbMixin):
 
     def _get_dvr_fip_host_routes_by_bgp_speaker(self, context,
                                                 bgp_speaker_id):
+        router_attrs = l3_attrs_db.RouterExtraAttributes
         with context.session.begin(subtransactions=True):
             gw_query = self._get_gateway_query(context, bgp_speaker_id)
             fip_query = self._get_fip_query(context, bgp_speaker_id)
 
+            fip_query = fip_query.filter(
+                l3_db.FloatingIP.router_id == router_attrs.router_id,
+                router_attrs.distributed == sa.sql.true())
+
             #Create the join query
             join_query = self._join_fip_by_host_binding_to_agent_gateway(
                 context,
