@@ -107,6 +107,12 @@ class DbBasePluginCommon(common_db_mixin.CommonDbMixin):
             subnet_id=subnet_id
         )
         context.session.add(allocated)
+        # NOTE(kevinbenton): We add this to the session info so the sqlalchemy
+        # object isn't immediately garbage collected. Otherwise when the
+        # fixed_ips relationship is referenced a new persistent object will be
+        # added to the session that will interfere with retry operations.
+        # See bug 1556178 for details.
+        context.session.info.setdefault('allocated_ips', []).append(allocated)
 
     def _make_subnet_dict(self, subnet, fields=None, context=None):
         res = {'id': subnet['id'],
@@ -130,7 +136,7 @@ class DbBasePluginCommon(common_db_mixin.CommonDbMixin):
                                for route in subnet['routes']],
                }
         # The shared attribute for a subnet is the same as its parent network
-        res['shared'] = self._is_network_shared(context, subnet.networks)
+        res['shared'] = self._is_network_shared(context, subnet.rbac_entries)
         # Call auxiliary extend functions, if any
         self._apply_dict_extend_functions(attributes.SUBNETS, res, subnet)
         return self._fields(res, fields)
@@ -271,18 +277,18 @@ class DbBasePluginCommon(common_db_mixin.CommonDbMixin):
                'status': network['status'],
                'subnets': [subnet['id']
                            for subnet in network['subnets']]}
-        res['shared'] = self._is_network_shared(context, network)
+        res['shared'] = self._is_network_shared(context, network.rbac_entries)
         # Call auxiliary extend functions, if any
         if process_extensions:
             self._apply_dict_extend_functions(
                 attributes.NETWORKS, res, network)
         return self._fields(res, fields)
 
-    def _is_network_shared(self, context, network):
+    def _is_network_shared(self, context, rbac_entries):
         # The shared attribute for a network now reflects if the network
         # is shared to the calling tenant via an RBAC entry.
         matches = ('*',) + ((context.tenant_id,) if context else ())
-        for entry in network.rbac_entries:
+        for entry in rbac_entries:
             if (entry.action == 'access_as_shared' and
                     entry.target_tenant in matches):
                 return True
