@@ -13,79 +13,23 @@
 #    under the License.
 
 from oslo_log import log as logging
-from oslo_serialization import jsonutils
-from oslo_utils import importutils
 from oslo_utils import uuidutils
 import sqlalchemy as sa
 from sqlalchemy import orm
 from sqlalchemy.orm import exc as sa_exc
 
-from neutron.common import exceptions as qexception
+from neutron.api.v2 import attributes as attr
 from neutron.db import common_db_mixin
 from neutron.db import model_base
-from neutron.db import models_v2
-from neutron.plugins.common import constants
-
+from neutron.db import servicetype_db as sdb
+from neutron.extensions import flavors as ext_flavors
 
 LOG = logging.getLogger(__name__)
 
 
-# Flavor Exceptions
-class FlavorNotFound(qexception.NotFound):
-    message = _("Flavor %(flavor_id)s could not be found")
-
-
-class FlavorInUse(qexception.InUse):
-    message = _("Flavor %(flavor_id)s is used by some service instance")
-
-
-class ServiceProfileNotFound(qexception.NotFound):
-    message = _("Service Profile %(sp_id)s could not be found")
-
-
-class ServiceProfileInUse(qexception.InUse):
-    message = _("Service Profile %(sp_id)s is used by some service instance")
-
-
-class FlavorServiceProfileBindingExists(qexception.Conflict):
-    message = _("Service Profile %(sp_id)s is already associated "
-                "with flavor %(fl_id)s")
-
-
-class FlavorServiceProfileBindingNotFound(qexception.NotFound):
-    message = _("Service Profile %(sp_id)s is not associated "
-                "with flavor %(fl_id)s")
-
-
-class DummyCorePlugin(object):
-    pass
-
-
-class DummyServicePlugin(object):
-
-    def driver_loaded(self, driver, service_profile):
-        pass
-
-    def get_plugin_type(self):
-        return constants.DUMMY
-
-    def get_plugin_description(self):
-        return "Dummy service plugin, aware of flavors"
-
-
-class DummyServiceDriver(object):
-
-    @staticmethod
-    def get_service_type():
-        return constants.DUMMY
-
-    def __init__(self, plugin):
-        pass
-
-
-class Flavor(model_base.BASEV2, models_v2.HasId):
-    name = sa.Column(sa.String(255))
-    description = sa.Column(sa.String(1024))
+class Flavor(model_base.BASEV2, model_base.HasId):
+    name = sa.Column(sa.String(attr.NAME_MAX_LEN))
+    description = sa.Column(sa.String(attr.LONG_DESCRIPTION_MAX_LEN))
     enabled = sa.Column(sa.Boolean, nullable=False, default=True,
                         server_default=sa.sql.true())
     # Make it True for multi-type flavors
@@ -94,8 +38,8 @@ class Flavor(model_base.BASEV2, models_v2.HasId):
         cascade="all, delete-orphan")
 
 
-class ServiceProfile(model_base.BASEV2, models_v2.HasId):
-    description = sa.Column(sa.String(1024))
+class ServiceProfile(model_base.BASEV2, model_base.HasId):
+    description = sa.Column(sa.String(attr.LONG_DESCRIPTION_MAX_LEN))
     driver = sa.Column(sa.String(1024), nullable=False)
     enabled = sa.Column(sa.Boolean, nullable=False, default=True,
                         server_default=sa.sql.true())
@@ -116,36 +60,21 @@ class FlavorServiceProfileBinding(model_base.BASEV2):
     service_profile = orm.relationship(ServiceProfile)
 
 
-class FlavorManager(common_db_mixin.CommonDbMixin):
+class FlavorsDbMixin(common_db_mixin.CommonDbMixin):
+
     """Class to support flavors and service profiles."""
 
-    supported_extension_aliases = ["flavors"]
-
-    def __init__(self, manager=None):
-        # manager = None is UT usage where FlavorManager is loaded as
-        # a core plugin
-        self.manager = manager
-
-    def get_plugin_name(self):
-        return constants.FLAVORS
-
-    def get_plugin_type(self):
-        return constants.FLAVORS
-
-    def get_plugin_description(self):
-        return "Neutron Flavors and Service Profiles manager plugin"
-
     def _get_flavor(self, context, flavor_id):
         try:
             return self._get_by_id(context, Flavor, flavor_id)
         except sa_exc.NoResultFound:
-            raise FlavorNotFound(flavor_id=flavor_id)
+            raise ext_flavors.FlavorNotFound(flavor_id=flavor_id)
 
     def _get_service_profile(self, context, sp_id):
         try:
             return self._get_by_id(context, ServiceProfile, sp_id)
         except sa_exc.NoResultFound:
-            raise ServiceProfileNotFound(sp_id=sp_id)
+            raise ext_flavors.ServiceProfileNotFound(sp_id=sp_id)
 
     def _make_flavor_dict(self, flavor_db, fields=None):
         res = {'id': flavor_db['id'],
@@ -178,12 +107,21 @@ class FlavorManager(common_db_mixin.CommonDbMixin):
         pass
 
     def _ensure_service_profile_not_in_use(self, context, sp_id):
-        # Future TODO(enikanorov): check that there is no binding to instances
-        # and no binding to flavors. Shall be addressed in future
+        """Ensures no current bindings to flavors exist."""
         fl = (context.session.query(FlavorServiceProfileBinding).
               filter_by(service_profile_id=sp_id).first())
         if fl:
-            raise ServiceProfileInUse(sp_id=sp_id)
+            raise ext_flavors.ServiceProfileInUse(sp_id=sp_id)
+
+    def _validate_driver(self, context, driver):
+        """Confirms a non-empty driver is a valid provider."""
+        service_type_manager = sdb.ServiceTypeManager.get_instance()
+        providers = service_type_manager.get_service_providers(
+            context,
+            filters={'driver': driver})
+
+        if not providers:
+            raise ext_flavors.ServiceProfileDriverNotFound(driver=driver)
 
     def create_flavor(self, context, flavor):
         fl = flavor['flavor']
@@ -202,7 +140,6 @@ class FlavorManager(common_db_mixin.CommonDbMixin):
             self._ensure_flavor_not_in_use(context, flavor_id)
             fl_db = self._get_flavor(context, flavor_id)
             fl_db.update(fl)
-
         return self._make_flavor_dict(fl_db)
 
     def get_flavor(self, context, flavor_id, fields=None):
@@ -231,15 +168,14 @@ class FlavorManager(common_db_mixin.CommonDbMixin):
             binding = bind_qry.filter_by(service_profile_id=sp['id'],
                                          flavor_id=flavor_id).first()
             if binding:
-                raise FlavorServiceProfileBindingExists(
+                raise ext_flavors.FlavorServiceProfileBindingExists(
                     sp_id=sp['id'], fl_id=flavor_id)
             binding = FlavorServiceProfileBinding(
                 service_profile_id=sp['id'],
                 flavor_id=flavor_id)
             context.session.add(binding)
         fl_db = self._get_flavor(context, flavor_id)
-        sps = [x['service_profile_id'] for x in fl_db.service_profiles]
-        return sps
+        return self._make_flavor_dict(fl_db)
 
     def delete_flavor_service_profile(self, context,
                                       service_profile_id, flavor_id):
@@ -248,7 +184,7 @@ class FlavorManager(common_db_mixin.CommonDbMixin):
                        filter_by(service_profile_id=service_profile_id,
                        flavor_id=flavor_id).first())
             if not binding:
-                raise FlavorServiceProfileBindingNotFound(
+                raise ext_flavors.FlavorServiceProfileBindingNotFound(
                     sp_id=service_profile_id, fl_id=flavor_id)
             context.session.delete(binding)
 
@@ -259,55 +195,21 @@ class FlavorManager(common_db_mixin.CommonDbMixin):
                        filter_by(service_profile_id=service_profile_id,
                        flavor_id=flavor_id).first())
             if not binding:
-                raise FlavorServiceProfileBindingNotFound(
+                raise ext_flavors.FlavorServiceProfileBindingNotFound(
                     sp_id=service_profile_id, fl_id=flavor_id)
         res = {'service_profile_id': service_profile_id,
                'flavor_id': flavor_id}
         return self._fields(res, fields)
 
-    def _load_dummy_driver(self, driver):
-        driver = DummyServiceDriver
-        driver_klass = driver
-        return driver_klass
-
-    def _load_driver(self, profile):
-        driver_klass = importutils.import_class(profile.driver)
-        return driver_klass
-
     def create_service_profile(self, context, service_profile):
         sp = service_profile['service_profile']
-        with context.session.begin(subtransactions=True):
-            driver_klass = self._load_dummy_driver(sp['driver'])
-            # 'get_service_type' must be a static method so it can't be changed
-            svc_type = DummyServiceDriver.get_service_type()
 
-            sp_db = ServiceProfile(id=uuidutils.generate_uuid(),
-                                   description=sp['description'],
-                                   driver=svc_type,
-                                   enabled=sp['enabled'],
-                                   metainfo=jsonutils.dumps(sp['metainfo']))
-            context.session.add(sp_db)
-        try:
-            # driver_klass = self._load_dummy_driver(sp_db)
-            # Future TODO(madhu_ak): commented for now to load dummy driver
-            # until there is flavor supported driver
-            # plugin = self.manager.get_service_plugins()[svc_type]
-            # plugin.driver_loaded(driver_klass(plugin), sp_db)
-            # svc_type = DummyServiceDriver.get_service_type()
-            # plugin = self.manager.get_service_plugins()[svc_type]
-            # plugin = FlavorManager(manager.NeutronManager().get_instance())
-            # plugin = DummyServicePlugin.get_plugin_type(svc_type)
-            plugin = DummyServicePlugin()
-            plugin.driver_loaded(driver_klass(svc_type), sp_db)
-        except Exception:
-            # Future TODO(enikanorov): raise proper exception
-            self.delete_service_profile(context, sp_db['id'])
-            raise
-        return self._make_service_profile_dict(sp_db)
+        if sp['driver']:
+            self._validate_driver(context, sp['driver'])
+        else:
+            if not sp['metainfo']:
+                raise ext_flavors.ServiceProfileEmpty()
 
-    def unit_create_service_profile(self, context, service_profile):
-        # Note: Triggered by unit tests pointing to dummy driver
-        sp = service_profile['service_profile']
         with context.session.begin(subtransactions=True):
             sp_db = ServiceProfile(id=uuidutils.generate_uuid(),
                                    description=sp['description'],
@@ -315,21 +217,16 @@ class FlavorManager(common_db_mixin.CommonDbMixin):
                                    enabled=sp['enabled'],
                                    metainfo=sp['metainfo'])
             context.session.add(sp_db)
-        try:
-            driver_klass = self._load_driver(sp_db)
-            # require get_service_type be a static method
-            svc_type = driver_klass.get_service_type()
-            plugin = self.manager.get_service_plugins()[svc_type]
-            plugin.driver_loaded(driver_klass(plugin), sp_db)
-        except Exception:
-            # Future TODO(enikanorov): raise proper exception
-            self.delete_service_profile(context, sp_db['id'])
-            raise
+
         return self._make_service_profile_dict(sp_db)
 
     def update_service_profile(self, context,
                                service_profile_id, service_profile):
         sp = service_profile['service_profile']
+
+        if sp.get('driver'):
+            self._validate_driver(context, sp['driver'])
+
         with context.session.begin(subtransactions=True):
             self._ensure_service_profile_not_in_use(context,
                                                     service_profile_id)
@@ -356,3 +253,41 @@ class FlavorManager(common_db_mixin.CommonDbMixin):
                                     sorts=sorts, limit=limit,
                                     marker_obj=marker,
                                     page_reverse=page_reverse)
+
+    def get_flavor_next_provider(self, context, flavor_id,
+                                 filters=None, fields=None,
+                                 sorts=None, limit=None,
+                                 marker=None, page_reverse=False):
+        """From flavor, choose service profile and find provider for driver."""
+
+        with context.session.begin(subtransactions=True):
+            bind_qry = context.session.query(FlavorServiceProfileBinding)
+            binding = bind_qry.filter_by(flavor_id=flavor_id).first()
+            if not binding:
+                raise ext_flavors.FlavorServiceProfileBindingNotFound(
+                    sp_id='', fl_id=flavor_id)
+
+        # Get the service profile from the first binding
+        # TODO(jwarendt) Should become a scheduling framework instead
+        sp_db = self._get_service_profile(context,
+                                          binding['service_profile_id'])
+
+        if not sp_db.enabled:
+            raise ext_flavors.ServiceProfileDisabled()
+
+        LOG.debug("Found driver %s.", sp_db.driver)
+
+        service_type_manager = sdb.ServiceTypeManager.get_instance()
+        providers = service_type_manager.get_service_providers(
+            context,
+            filters={'driver': sp_db.driver})
+
+        if not providers:
+            raise ext_flavors.ServiceProfileDriverNotFound(driver=sp_db.driver)
+
+        LOG.debug("Found providers %s.", providers)
+
+        res = {'driver': sp_db.driver,
+               'provider': providers[0].get('name')}
+
+        return [self._fields(res, fields)]
