@@ -372,21 +372,28 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
     def _get_changed_ips_for_port(self, context, original_ips,
                                   new_ips, device_owner):
         """Calculate changes in IPs for the port."""
+        # Collect auto addressed subnet ids that has to be removed on update
+        delete_subnet_ids = set(ip['subnet_id'] for ip in new_ips
+                                if ip.get('delete_subnet'))
+        ips = [ip for ip in new_ips
+               if ip.get('subnet_id') not in delete_subnet_ids]
         # the new_ips contain all of the fixed_ips that are to be updated
-        self._validate_max_ips_per_port(new_ips, device_owner)
+        self._validate_max_ips_per_port(ips, device_owner)
 
         add_ips = []
         remove_ips = []
+
         ips_map = {ip['ip_address']: ip
                    for ip in itertools.chain(new_ips, original_ips)
                    if 'ip_address' in ip}
 
         new = set()
         for ip in new_ips:
-            if 'ip_address' in ip:
-                new.add(ip['ip_address'])
-            else:
-                add_ips.append(ip)
+            if ip.get('subnet_id') not in delete_subnet_ids:
+                if 'ip_address' in ip:
+                    new.add(ip['ip_address'])
+                else:
+                    add_ips.append(ip)
 
         # Convert original ip addresses to sets
         orig = set(ip['ip_address'] for ip in original_ips)
@@ -401,11 +408,13 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
 
         # Mark ip for removing if it is not found in new_ips
         # and subnet requires ip to be set manually.
-        # For auto addresses leave ip unchanged
+        # For auto addressed subnet leave ip unchanged
+        # unless it is explicitly marked for delete.
         for ip in remove:
             subnet_id = ips_map[ip]['subnet_id']
-            if self._is_ip_required_by_subnet(context, subnet_id,
-                                              device_owner):
+            ip_required = self._is_ip_required_by_subnet(context, subnet_id,
+                                                         device_owner)
+            if ip_required or subnet_id in delete_subnet_ids:
                 remove_ips.append(ips_map[ip])
             else:
                 prev_ips.append(ips_map[ip])
