@@ -31,6 +31,7 @@ from neutron.common import constants
 from neutron.common import exceptions as n_exc
 from neutron.common import utils as n_utils
 from neutron.db import agents_db
+from neutron.db import api as db_api
 from neutron.db.availability_zone import router as router_az_db
 from neutron.db import common_db_mixin
 from neutron.db import l3_attrs_db
@@ -511,6 +512,12 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
                 msg = _('Cannot change HA attribute of active routers. Please '
                         'set router admin_state_up to False prior to upgrade.')
                 raise n_exc.BadRequest(resource='router', msg=msg)
+
+            if requested_ha_state:
+                # This will throw HANotEnoughAvailableAgents if there aren't
+                # enough l3 agents to handle this router.
+                self.get_number_of_agents_for_scheduling(context)
+
             # set status to ALLOCATING so this router is no longer
             # provided to agents while its interfaces are being re-configured.
             # Keep in mind that if we want conversion to be hitless, this
@@ -759,3 +766,16 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
                     n_exc.PortNotFound):
                 # Take concurrently deleted interfaces in to account
                 pass
+
+
+def is_ha_router_port(device_owner, router_id):
+    session = db_api.get_session()
+    if device_owner in [constants.DEVICE_OWNER_ROUTER_INTF,
+                        constants.DEVICE_OWNER_ROUTER_SNAT]:
+        query = session.query(l3_attrs_db.RouterExtraAttributes)
+        query = query.filter_by(ha=True)
+        query = query.filter(l3_attrs_db.RouterExtraAttributes.router_id ==
+                             router_id)
+        return bool(query.limit(1).count())
+    else:
+        return False
