@@ -12,16 +12,23 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from neutron_lib import constants
 from sqlalchemy import func
 from sqlalchemy import sql
 
+from neutron.callbacks import events
+from neutron.callbacks import registry
+from neutron.callbacks import resources
 from neutron.db import agents_db
 from neutron.db import l3_agentschedulers_db as l3_sch_db
 from neutron.db import l3_attrs_db
 from neutron.db import l3_db
+from neutron.extensions import portbindings
+from neutron import manager
+from neutron.plugins.common import constants as service_constants
 
 
-class L3_HA_scheduler_db_mixin(l3_sch_db.L3AgentSchedulerDbMixin):
+class L3_HA_scheduler_db_mixin(l3_sch_db.AZL3AgentSchedulerDbMixin):
 
     def get_ha_routers_l3_agents_count(self, context):
         """Return a map between HA routers and how many agents every
@@ -43,10 +50,11 @@ class L3_HA_scheduler_db_mixin(l3_sch_db.L3AgentSchedulerDbMixin):
             filter(l3_attrs_db.RouterExtraAttributes.ha == sql.true()).
             group_by(binding_model.router_id).subquery())
 
-        query = (context.session.query(
-                 l3_db.Router.id, l3_db.Router.tenant_id, sub_query.c.count).
+        query = (context.session.query(l3_db.Router, sub_query.c.count).
                  join(sub_query))
-        return query
+
+        return [(self._make_router_dict(router), agent_count)
+                for router, agent_count in query]
 
     def get_l3_agents_ordered_by_num_routers(self, context, agent_ids):
         if not agent_ids:
@@ -80,3 +88,30 @@ class L3_HA_scheduler_db_mixin(l3_sch_db.L3AgentSchedulerDbMixin):
                 bindings = [(binding.l3_agent, None) for binding in bindings]
 
         return self._get_agents_dict_for_router(bindings)
+
+
+def _notify_l3_agent_ha_port_update(resource, event, trigger, **kwargs):
+    # HA router on a agent has to spawn keepalived only when HA network port
+    # is active. So notify agent when HA network port becomes active.
+    # 'update_device_up' will be set only when port status changed to active.
+    if not kwargs.get('update_device_up'):
+        return
+    port_db = kwargs.get('port')
+    context = kwargs.get('context')
+    core_plugin = manager.NeutronManager.get_plugin()
+    new_port = core_plugin._make_port_dict(port_db)
+    host = new_port.get(portbindings.HOST_ID)
+
+    if new_port and host:
+        new_device_owner = new_port.get('device_owner', '')
+        if (new_device_owner == constants.DEVICE_OWNER_ROUTER_HA_INTF and
+            new_port['status'] == constants.PORT_STATUS_ACTIVE):
+            l3plugin = manager.NeutronManager.get_service_plugins().get(
+                service_constants.L3_ROUTER_NAT)
+            l3plugin.l3_rpc_notifier.routers_updated_on_host(
+                context, [new_port['device_id']], host)
+
+
+def subscribe():
+    registry.subscribe(
+        _notify_l3_agent_ha_port_update, resources.PORT, events.AFTER_UPDATE)
