@@ -25,6 +25,9 @@ from neutron.db import address_scope_db  # noqa
 from neutron.db import agents_db  # noqa
 from neutron.db import agentschedulers_db  # noqa
 from neutron.db import allowedaddresspairs_db  # noqa
+from neutron.db import bgp_db  # noqa
+from neutron.db import bgp_dragentscheduler_db  # noqa
+from neutron.db import dns_db  # noqa
 from neutron.db import dvr_mac_db  # noqa
 from neutron.db import external_net_db  # noqa
 from neutron.db import extradhcpopt_db  # noqa
@@ -46,19 +49,15 @@ from neutron.db.quota import models  # noqa
 from neutron.db import rbac_db_models  # noqa
 from neutron.db import securitygroups_db  # noqa
 from neutron.db import servicetype_db  # noqa
+from neutron.db import tag_db  # noqa
 from neutron.ipam.drivers.neutrondb_ipam import db_models  # noqa
-from neutron.plugins.bigswitch.db import consistency_db  # noqa
-from neutron.plugins.bigswitch import routerrule_db  # noqa
-from neutron.plugins.brocade.db import models as brocade_models  # noqa
-from neutron.plugins.ml2.drivers.brocade.db import (  # noqa
-    models as ml2_brocade_models)
 from neutron.plugins.ml2.drivers import type_flat  # noqa
 from neutron.plugins.ml2.drivers import type_geneve  # noqa
 from neutron.plugins.ml2.drivers import type_gre  # noqa
 from neutron.plugins.ml2.drivers import type_vlan  # noqa
 from neutron.plugins.ml2.drivers import type_vxlan  # noqa
 from neutron.plugins.ml2 import models  # noqa
-from neutron.plugins.nuage import nuage_models  # noqa
+from neutron.services.auto_allocate import models  # noqa
 
 
 def get_metadata():
