@@ -121,7 +121,8 @@ class Port(model_base.HasStandardAttributes, model_base.BASEV2,
     network_id = sa.Column(sa.String(36), sa.ForeignKey("networks.id"),
                            nullable=False)
     fixed_ips = orm.relationship(IPAllocation, backref='port', lazy='joined',
-                                 passive_deletes='all')
+                                 cascade='all, delete-orphan')
+
     mac_address = sa.Column(sa.String(32), nullable=False)
     admin_state_up = sa.Column(sa.Boolean(), nullable=False)
     status = sa.Column(sa.String(16), nullable=False)
@@ -219,7 +220,7 @@ class Subnet(model_base.HasStandardAttributes, model_base.BASEV2,
     # subnets don't have their own rbac_entries, they just inherit from
     # the network rbac entries
     rbac_entries = orm.relationship(
-        rbac_db_models.NetworkRBAC, lazy='joined',
+        rbac_db_models.NetworkRBAC, lazy='subquery', uselist=True,
         foreign_keys='Subnet.network_id',
         primaryjoin='Subnet.network_id==NetworkRBAC.object_id')
 
@@ -267,14 +268,14 @@ class Network(model_base.HasStandardAttributes, model_base.BASEV2,
     name = sa.Column(sa.String(attr.NAME_MAX_LEN))
     ports = orm.relationship(Port, backref='networks')
     subnets = orm.relationship(
-        Subnet, backref=orm.backref('networks', lazy='joined'),
-        lazy="joined")
+        Subnet, backref=orm.backref('networks', lazy='subquery'),
+        lazy="subquery")
     status = sa.Column(sa.String(16))
     admin_state_up = sa.Column(sa.Boolean)
     mtu = sa.Column(sa.Integer, nullable=True)
     vlan_transparent = sa.Column(sa.Boolean, nullable=True)
     rbac_entries = orm.relationship(rbac_db_models.NetworkRBAC,
-                                    backref='network', lazy='joined',
+                                    backref='network', lazy='subquery',
                                     cascade='all, delete, delete-orphan')
     availability_zone_hints = sa.Column(sa.String(255))
     dhcp_agents = orm.relationship(
