@@ -43,13 +43,12 @@ class RbacPluginMixin(common_db_mixin.CommonDbMixin):
         except c_exc.CallbackFailure as e:
             raise n_exc.InvalidInput(error_message=e)
         dbmodel = models.get_type_model_map()[e['object_type']]
-        tenant_id = self._get_tenant_id_for_create(context, e)
         try:
             with context.session.begin(subtransactions=True):
                 db_entry = dbmodel(object_id=e['object_id'],
                                    target_tenant=e['target_tenant'],
                                    action=e['action'],
-                                   tenant_id=tenant_id)
+                                   tenant_id=e['tenant_id'])
                 context.session.add(db_entry)
         except db_exc.DBDuplicateEntry:
             raise ext_rbac.DuplicateRbacPolicy()
@@ -105,11 +104,21 @@ class RbacPluginMixin(common_db_mixin.CommonDbMixin):
 
     def get_rbac_policies(self, context, filters=None, fields=None,
                           sorts=None, limit=None, page_reverse=False):
-        model = common_db_mixin.UnionModel(
-            models.get_type_model_map(), 'object_type')
-        return self._get_collection(
-            context, model, self._make_rbac_policy_dict, filters=filters,
-            fields=fields, sorts=sorts, limit=limit, page_reverse=page_reverse)
+        filters = filters or {}
+        object_type_filters = filters.pop('object_type', None)
+        models_to_query = [
+            m for t, m in models.get_type_model_map().items()
+            if object_type_filters is None or t in object_type_filters
+        ]
+        collections = [self._get_collection(
+            context, model, self._make_rbac_policy_dict,
+            filters=filters, fields=fields, sorts=sorts,
+            limit=limit, page_reverse=page_reverse)
+            for model in models_to_query]
+        # NOTE(kevinbenton): we don't have to worry about pagination,
+        # limits, or page_reverse currently because allow_pagination is
+        # set to False in 'neutron.extensions.rbac'
+        return [item for c in collections for item in c]
 
     def _get_object_type(self, context, entry_id):
         """Scans all RBAC tables for an ID to figure out the type.
