@@ -14,9 +14,9 @@
 
 import abc
 
-from oslo_log import log as logging
 import six
 
+from neutron._i18n import _
 from neutron.api import extensions
 from neutron.api.v2 import attributes as attr
 from neutron.api.v2 import resource_helper
@@ -24,8 +24,6 @@ from neutron.common import exceptions as nexception
 from neutron.plugins.common import constants
 from neutron.services import service_base
 
-LOG = logging.getLogger(__name__)
-
 
 class MeteringLabelNotFound(nexception.NotFound):
     message = _("Metering label %(label_id)s does not exist")
