@@ -18,6 +18,9 @@ from neutron.api.v2 import attributes
 from neutron.common import exceptions as nexception
 
 
+DEFAULT_PORT_SECURITY = True
+
+
 class PortSecurityPortHasSecurityGroup(nexception.InUse):
     message = _("Port has security group associated. Cannot disable port "
                 "security or ip address until security group is removed")
@@ -28,16 +31,13 @@ class PortSecurityAndIPRequiredForSecurityGroups(nexception.InvalidInput):
                 " address in order to use security groups.")
 
 
-class PortSecurityBindingNotFound(nexception.InvalidExtensionEnv):
-    message = _("Port does not have port security binding.")
-
 PORTSECURITY = 'port_security_enabled'
 EXTENDED_ATTRIBUTES_2_0 = {
     'networks': {
         PORTSECURITY: {'allow_post': True, 'allow_put': True,
                        'convert_to': attributes.convert_to_boolean,
                        'enforce_policy': True,
-                       'default': True,
+                       'default': DEFAULT_PORT_SECURITY,
                        'is_visible': True},
     },
     'ports': {
