@@ -59,12 +59,12 @@ RESOURCE_ATTRIBUTE_MAP = {
                         'enforce_policy': True},
         'object_id': {'allow_post': True, 'allow_put': False,
                       'validate': {'type:uuid': None},
-                      'is_visible': True, 'default': None,
-                      'enforce_policy': True},
+                      'is_visible': True, 'enforce_policy': True},
         'target_tenant': {'allow_post': True, 'allow_put': True,
-                          'is_visible': True, 'enforce_policy': True,
-                          'default': None},
+                          'validate': {'type:string': attr.TENANT_ID_MAX_LEN},
+                          'is_visible': True, 'enforce_policy': True},
         'tenant_id': {'allow_post': True, 'allow_put': False,
+                      'validate': {'type:string': attr.TENANT_ID_MAX_LEN},
                       'required_by_policy': True, 'is_visible': True},
         'action': {'allow_post': True, 'allow_put': False,
                    # action depends on type so validation has to occur in
@@ -72,8 +72,7 @@ RESOURCE_ATTRIBUTE_MAP = {
                    'validate': {'type:string': attr.DESCRIPTION_MAX_LEN},
                    # we set enforce_policy so operators can define policies
                    # that restrict actions
-                   'is_visible': True, 'enforce_policy': True,
-                   'default': None},
+                   'is_visible': True, 'enforce_policy': True}
     }
 }
 
