@@ -20,6 +20,7 @@ import netaddr
 from oslo_db import exception as db_exc
 from oslo_utils import uuidutils
 
+from neutron._i18n import _
 from neutron.api.v2 import attributes
 from neutron.common import constants
 from neutron.common import exceptions as n_exc
@@ -58,7 +59,7 @@ class SubnetAllocator(driver.Pool):
 
         # NOTE(cbrandily): the update disallows 2 concurrent subnet allocation
         # to succeed: at most 1 transaction will succeed, others will be
-        # rollbacked and be caught in neutron.db.v2.base
+        # rolled back and be caught in neutron.db.v2.base
         query = self._context.session.query(models_v2.SubnetPool).filter_by(
             id=self._subnetpool['id'], hash=current_hash)
         count = query.update({'hash': new_hash})
@@ -227,7 +228,8 @@ class SubnetPoolReader(object):
         self._read_id(subnetpool)
         self._read_prefix_bounds(subnetpool)
         self._read_attrs(subnetpool,
-                         ['tenant_id', 'name', 'shared'])
+                         ['tenant_id', 'name', 'is_default', 'shared'])
+        self.description = subnetpool.get('description')
         self._read_address_scope(subnetpool)
         self.subnetpool = {'id': self.id,
                            'name': self.name,
@@ -241,7 +243,9 @@ class SubnetPoolReader(object):
                            'default_prefixlen': self.default_prefixlen,
                            'default_quota': self.default_quota,
                            'address_scope_id': self.address_scope_id,
-                           'shared': self.shared}
+                           'is_default': self.is_default,
+                           'shared': self.shared,
+                           'description': self.description}
 
     def _read_attrs(self, subnetpool, keys):
         for key in keys:
