@@ -21,6 +21,7 @@ methods that needs to be implemented by a v2 Neutron Plug-in.
 """
 
 import abc
+
 import six
 
 
@@ -83,7 +84,7 @@ class NeutronPluginBaseV2(object):
         :param filters: a dictionary with keys that are valid keys for
                         a subnet as listed in the :obj:`RESOURCE_ATTRIBUTE_MAP`
                         object in :file:`neutron/api/v2/attributes.py`.
-                        Values in this dictiontary are an iterable containing
+                        Values in this dictionary are an iterable containing
                         values that will be used for an exact match comparison
                         for that value.  Each result returned by this
                         function will have matched one of the values for each
@@ -108,7 +109,7 @@ class NeutronPluginBaseV2(object):
                         a network as listed in the
                         :obj:`RESOURCE_ATTRIBUTE_MAP` object in
                         :file:`neutron/api/v2/attributes.py`.  Values in this
-                        dictiontary are an iterable containing values that
+                        dictionary are an iterable containing values that
                         will be used for an exact match comparison for that
                         value.  Each result returned by this function will
                         have matched one of the values for each key in filters.
@@ -224,7 +225,7 @@ class NeutronPluginBaseV2(object):
                         a network as listed in the
                         :obj:`RESOURCE_ATTRIBUTE_MAP` object in
                         :file:`neutron/api/v2/attributes.py`.  Values in this
-                        dictiontary are an iterable containing values that will
+                        dictionary are an iterable containing values that will
                         be used for an exact match comparison for that value.
                         Each result returned by this function will have matched
                         one of the values for each key in filters.
@@ -248,7 +249,7 @@ class NeutronPluginBaseV2(object):
                         a network as listed in the
                         :obj:`RESOURCE_ATTRIBUTE_MAP` object
                         in :file:`neutron/api/v2/attributes.py`. Values in
-                        this dictiontary are an iterable containing values that
+                        this dictionary are an iterable containing values that
                         will be used for an exact match comparison for that
                         value.  Each result returned by this function will have
                         matched one of the values for each key in filters.
@@ -321,7 +322,7 @@ class NeutronPluginBaseV2(object):
         :param filters: a dictionary with keys that are valid keys for
                         a port as listed in the  :obj:`RESOURCE_ATTRIBUTE_MAP`
                         object in :file:`neutron/api/v2/attributes.py`. Values
-                        in this dictiontary are an iterable containing values
+                        in this dictionary are an iterable containing values
                         that will be used for an exact match comparison for
                         that value.  Each result returned by this function will
                         have matched one of the values for each key in filters.
@@ -344,7 +345,7 @@ class NeutronPluginBaseV2(object):
                         a network as listed in the
                         :obj:`RESOURCE_ATTRIBUTE_MAP` object in
                         :file:`neutron/api/v2/attributes.py`.  Values in this
-                        dictiontary are an iterable containing values that will
+                        dictionary are an iterable containing values that will
                         be used for an exact match comparison for that value.
                         Each result returned by this function will have matched
                         one of the values for each key in filters.
@@ -375,6 +376,16 @@ class NeutronPluginBaseV2(object):
         """
         raise NotImplementedError()
 
+    def start_rpc_state_reports_listener(self):
+        """Start the RPC listeners consuming state reports queue.
+
+        This optional method creates rpc consumer for REPORTS queue only.
+
+        .. note:: this method is optional, as it was not part of the originally
+                  defined plugin API.
+        """
+        raise NotImplementedError()
+
     def rpc_workers_supported(self):
         """Return whether the plugin supports multiple RPC workers.
 
@@ -390,6 +401,15 @@ class NeutronPluginBaseV2(object):
         return (self.__class__.start_rpc_listeners !=
                 NeutronPluginBaseV2.start_rpc_listeners)
 
+    def rpc_state_report_workers_supported(self):
+        """Return whether the plugin supports state report RPC workers.
+
+        .. note:: this method is optional, as it was not part of the originally
+                  defined plugin API.
+        """
+        return (self.__class__.start_rpc_state_reports_listener !=
+                NeutronPluginBaseV2.start_rpc_state_reports_listener)
+
     def get_workers(self):
         """Returns a collection NeutronWorker instances
 
