@@ -15,9 +15,11 @@
 
 from keystonemiddleware import auth_token
 from oslo_config import cfg
+from oslo_middleware import cors
 from oslo_middleware import request_id
 import pecan
 
+from neutron.api import versions
 from neutron.common import exceptions as n_exc
 from neutron.pecan_wsgi import hooks
 from neutron.pecan_wsgi import startup
@@ -44,12 +46,11 @@ def setup_app(*args, **kwargs):
     app_hooks = [
         hooks.ExceptionTranslationHook(),  # priority 100
         hooks.ContextHook(),  # priority 95
-        hooks.MemberActionHook(),  # piority 95
-        hooks.AttributePopulationHook(),  # priority 120
+        hooks.BodyValidationHook(),  # priority 120
         hooks.OwnershipValidationHook(),  # priority 125
         hooks.QuotaEnforcementHook(),  # priority 130
-        hooks.PolicyHook(),  # priority 135
-        hooks.NotifierHook(),  # priority 140
+        hooks.NotifierHook(),  # priority 135
+        hooks.PolicyHook(),  # priority 140
     ]
 
     app = pecan.make_app(
@@ -74,4 +75,23 @@ def _wrap_app(app):
     else:
         raise n_exc.InvalidConfigurationOption(
             opt_name='auth_strategy', opt_value=cfg.CONF.auth_strategy)
+
+    # version can be unauthenticated so it goes outside of auth
+    app = versions.Versions(app)
+
+    # This should be the last middleware in the list (which results in
+    # it being the first in the middleware chain). This is to ensure
+    # that any errors thrown by other middleware, such as an auth
+    # middleware - are annotated with CORS headers, and thus accessible
+    # by the browser.
+    app = cors.CORS(app, cfg.CONF)
+    app.set_latent(
+        allow_headers=['X-Auth-Token', 'X-Identity-Status', 'X-Roles',
+                       'X-Service-Catalog', 'X-User-Id', 'X-Tenant-Id',
+                       'X-OpenStack-Request-ID'],
+        allow_methods=['GET', 'PUT', 'POST', 'DELETE', 'PATCH'],
+        expose_headers=['X-Auth-Token', 'X-Subject-Token', 'X-Service-Token',
+                        'X-OpenStack-Request-ID']
+    )
+
     return app
