@@ -15,6 +15,7 @@
 
 """Exceptions used by ML2."""
 
+from neutron._i18n import _
 from neutron.common import exceptions
 
 
@@ -26,3 +27,14 @@ class MechanismDriverError(exceptions.NeutronException):
 class ExtensionDriverError(exceptions.InvalidInput):
     """Extension driver call failed."""
     message = _("Extension %(driver)s failed.")
+
+
+class ExtensionDriverNotFound(exceptions.InvalidConfigurationOption):
+    """Required extension driver not found in ML2 config."""
+    message = _("Extension driver %(driver)s required for "
+                "service plugin %(service_plugin)s not found.")
+
+
+class UnknownNetworkType(exceptions.NeutronException):
+    """Network with unknown type."""
+    message = _("Unknown network type %(network_type)s.")
