@@ -43,23 +43,16 @@ ml2_opts = [
     cfg.IntOpt('path_mtu', default=constants.DEFAULT_NETWORK_MTU,
                help=_('Maximum size of an IP packet (MTU) that can traverse '
                       'the underlying physical network infrastructure without '
-                      'fragmentation. For instances using a '
-                      'self-service/private network, neutron subtracts the '
-                      'overlay protocol overhead from this value and '
-                      'provides it to instances via DHCP option 26. For '
-                      'example, using a value of 9000, DHCP provides 8950 '
-                      'to instances using a VXLAN network that contains '
-                      '50 bytes of overhead. Using a value of 0 disables '
-                      'this feature and instances typically default to a '
-                      '1500 MTU. Only impacts instances, not neutron network '
-                      'components such as bridges and routers.')),
+                      'fragmentation for overlay/tunnel networks. In most '
+                      'cases, use the same value as the global_physnet_mtu '
+                      'option.')),
     cfg.ListOpt('physical_network_mtus',
                 default=[],
                 help=_("A list of mappings of physical networks to MTU "
                        "values. The format of the mapping is "
                        "<physnet>:<mtu val>. This mapping allows "
                        "specifying a physical network MTU value that "
-                       "differs from the default segment_mtu value.")),
+                       "differs from the default global_physnet_mtu value.")),
     cfg.StrOpt('external_network_type',
                help=_("Default network type for external networks when no "
                       "provider attributes are specified. By default it is "
