@@ -19,6 +19,7 @@ from oslo_config import cfg
 from oslo_db import exception as db_exc
 from oslo_log import log
 
+from neutron._i18n import _LE
 from neutron.common import exceptions as exc
 from neutron.common import utils
 from neutron.plugins.common import utils as p_utils
@@ -36,9 +37,10 @@ class BaseTypeDriver(api.TypeDriver):
     def __init__(self):
         try:
             self.physnet_mtus = utils.parse_mappings(
-                cfg.CONF.ml2.physical_network_mtus
+                cfg.CONF.ml2.physical_network_mtus, unique_values=False
             )
-        except Exception:
+        except Exception as e:
+            LOG.error(_LE("Failed to parse physical_network_mtus: %s"), e)
             self.physnet_mtus = []
 
     def get_mtu(self, physical_network=None):
