@@ -229,6 +229,10 @@ class OVSTunnelBridge(ovs_bridge.OVSAgentBridge,
                                         n_bits=32),
                    ofpp.OFPActionSetField(arp_sha=mac),
                    ofpp.OFPActionSetField(arp_spa=ip),
+                   ofpp.NXActionRegMove(src_field='eth_src',
+                                        dst_field='eth_dst',
+                                        n_bits=48),
+                   ofpp.OFPActionSetField(eth_src_nxm=mac),
                    ofpp.OFPActionOutput(ofp.OFPP_IN_PORT, 0)]
         self.install_apply_actions(table_id=constants.ARP_RESPONDER,
                                    priority=1,
