@@ -45,8 +45,9 @@ class OpenvswitchMechanismDriver(mech_agent.SimpleAgentMechanismDriverBase):
 
     def __init__(self):
         sg_enabled = securitygroups_rpc.is_firewall_enabled()
-        hybrid_plug_required = (cfg.CONF.SECURITYGROUP.firewall_driver in (
-            IPTABLES_FW_DRIVER_FULL, 'iptables_hybrid')) and sg_enabled
+        hybrid_plug_required = (not cfg.CONF.SECURITYGROUP.firewall_driver or
+            cfg.CONF.SECURITYGROUP.firewall_driver in (
+                IPTABLES_FW_DRIVER_FULL, 'iptables_hybrid')) and sg_enabled
         vif_details = {portbindings.CAP_PORT_FILTER: sg_enabled,
                        portbindings.OVS_HYBRID_PLUG: hybrid_plug_required}
         super(OpenvswitchMechanismDriver, self).__init__(
@@ -84,10 +85,16 @@ class OpenvswitchMechanismDriver(mech_agent.SimpleAgentMechanismDriverBase):
         return self.vif_type
 
     def get_vif_details(self, agent, context):
-        if (agent['configurations'].get('datapath_type') !=
-                a_const.OVS_DATAPATH_NETDEV):
-            return self.vif_details
-        caps = agent['configurations'].get('ovs_capabilities', {})
+        a_config = agent['configurations']
+        if a_config.get('datapath_type') != a_const.OVS_DATAPATH_NETDEV:
+            details = dict(self.vif_details)
+            hybrid = portbindings.OVS_HYBRID_PLUG
+            if hybrid in a_config:
+                # we only override the vif_details for hybrid pluggin set
+                # in the constuctor if the agent specifically requests it
+                details[hybrid] = a_config[hybrid]
+            return details
+        caps = a_config.get('ovs_capabilities', {})
         if a_const.OVS_DPDK_VHOST_USER in caps.get('iface_types', []):
             sock_path = self.agent_vhu_sockpath(agent, context.current['id'])
             return {
