@@ -27,9 +27,11 @@ from neutron.common import constants as n_const
 from neutron.common import exceptions
 from neutron.common import rpc as n_rpc
 from neutron.common import topics
+from neutron.db import l3_hamode_db
 from neutron.extensions import portbindings
 from neutron.extensions import portsecurity as psec
 from neutron import manager
+from neutron.plugins.ml2 import db as ml2_db
 from neutron.plugins.ml2 import driver_api as api
 from neutron.plugins.ml2.drivers import type_tunnel
 from neutron.services.qos import qos_consts
@@ -182,16 +184,18 @@ class RpcCallbacks(type_tunnel.TunnelRpcCallbackMixin):
             LOG.debug("Device %(device)s not bound to the"
                       " agent host %(host)s",
                       {'device': device, 'host': host})
-            return {'device': device,
-                    'exists': port_exists}
-
-        try:
-            port_exists = bool(plugin.update_port_status(
-                rpc_context, port_id, n_const.PORT_STATUS_DOWN, host))
-        except exc.StaleDataError:
-            port_exists = False
-            LOG.debug("delete_port and update_device_down are being executed "
-                      "concurrently. Ignoring StaleDataError.")
+        else:
+            try:
+                port_exists = bool(plugin.update_port_status(
+                    rpc_context, port_id, n_const.PORT_STATUS_DOWN, host))
+            except exc.StaleDataError:
+                port_exists = False
+                LOG.debug("delete_port and update_device_down are being "
+                          "executed concurrently. Ignoring StaleDataError.")
+                return {'device': device,
+                        'exists': port_exists}
+        self.notify_ha_port_status(port_id, rpc_context,
+                                   n_const.PORT_STATUS_DOWN, host)
 
         return {'device': device,
                 'exists': port_exists}
@@ -210,8 +214,13 @@ class RpcCallbacks(type_tunnel.TunnelRpcCallbackMixin):
             LOG.debug("Device %(device)s not bound to the"
                       " agent host %(host)s",
                       {'device': device, 'host': host})
-            return
+        else:
+            self.update_port_status_to_active(rpc_context, port_id, host)
+        self.notify_ha_port_status(port_id, rpc_context,
+                                   n_const.PORT_STATUS_ACTIVE, host)
 
+    def update_port_status_to_active(self, rpc_context, port_id, host):
+        plugin = manager.NeutronManager.get_plugin()
         port_id = plugin.update_port_status(rpc_context, port_id,
                                             n_const.PORT_STATUS_ACTIVE,
                                             host)
@@ -231,6 +240,28 @@ class RpcCallbacks(type_tunnel.TunnelRpcCallbackMixin):
             registry.notify(
                 resources.PORT, events.AFTER_UPDATE, plugin, **kwargs)
 
+    def notify_ha_port_status(self, port_id, rpc_context,
+                              status, host):
+        plugin = manager.NeutronManager.get_plugin()
+        l2pop_driver = plugin.mechanism_manager.mech_drivers.get(
+                'l2population')
+        if not l2pop_driver:
+            return
+        port = ml2_db.get_port(rpc_context.session, port_id)
+        if not port:
+            return
+        is_ha_port = l3_hamode_db.is_ha_router_port(port['device_owner'],
+                                                    port['device_id'])
+        if is_ha_port:
+            port_context = plugin.get_bound_port_context(
+                    rpc_context, port_id)
+            port_context.current['status'] = status
+            port_context.current[portbindings.HOST_ID] = host
+            if status == n_const.PORT_STATUS_ACTIVE:
+                l2pop_driver.obj.update_port_up(port_context)
+            else:
+                l2pop_driver.obj.update_port_down(port_context)
+
     def update_device_list(self, rpc_context, **kwargs):
         devices_up = []
         failed_devices_up = []
