@@ -14,15 +14,19 @@
 #    under the License.
 
 
+import collections
+import heapq
+
 from oslo_config import cfg
 from oslo_db import exception as db_exc
 from oslo_log import log as logging
 from sqlalchemy import sql
 
+from neutron._i18n import _LI, _LW
 from neutron.common import constants
 from neutron.db import agents_db
 from neutron.db import agentschedulers_db
-from neutron.i18n import _LI, _LW
+from neutron.extensions import availability_zone as az_ext
 from neutron.scheduler import base_resource_filter
 from neutron.scheduler import base_scheduler
 
@@ -55,7 +59,8 @@ class AutoScheduler(object):
             for dhcp_agent in dhcp_agents:
                 if agents_db.AgentDbMixin.is_agent_down(
                     dhcp_agent.heartbeat_timestamp):
-                    LOG.warn(_LW('DHCP agent %s is not active'), dhcp_agent.id)
+                    LOG.warning(_LW('DHCP agent %s is not active'),
+                                dhcp_agent.id)
                     continue
                 for net_id in net_ids:
                     agents = plugin.get_dhcp_agents_hosting_networks(
@@ -64,6 +69,12 @@ class AutoScheduler(object):
                         continue
                     if any(dhcp_agent.id == agent.id for agent in agents):
                         continue
+                    net = plugin.get_network(context, net_id)
+                    az_hints = (net.get(az_ext.AZ_HINTS) or
+                                cfg.CONF.default_availability_zones)
+                    if (az_hints and
+                        dhcp_agent['availability_zone'] not in az_hints):
+                        continue
                     bindings_to_add.append((dhcp_agent, net_id))
         # do it outside transaction so particular scheduling results don't
         # make other to fail
@@ -84,6 +95,46 @@ class WeightScheduler(base_scheduler.BaseWeightScheduler, AutoScheduler):
         super(WeightScheduler, self).__init__(DhcpFilter())
 
 
+class AZAwareWeightScheduler(WeightScheduler):
+
+    def select(self, plugin, context, resource_hostable_agents,
+               resource_hosted_agents, num_agents_needed):
+        """AZ aware scheduling
+           If the network has multiple AZs, agents are scheduled as
+           follows:
+           - select AZ with least agents scheduled for the network
+             (nondeterministic for AZs with same amount of agents scheduled)
+           - choose agent in the AZ with WeightScheduler
+        """
+        hostable_az_agents = collections.defaultdict(list)
+        num_az_agents = {}
+        for agent in resource_hostable_agents:
+            az_agent = agent['availability_zone']
+            hostable_az_agents[az_agent].append(agent)
+            if az_agent not in num_az_agents:
+                num_az_agents[az_agent] = 0
+        if num_agents_needed <= 0:
+            return []
+        for agent in resource_hosted_agents:
+            az_agent = agent['availability_zone']
+            if az_agent in num_az_agents:
+                num_az_agents[az_agent] += 1
+
+        num_az_q = [(value, key) for key, value in num_az_agents.items()]
+        heapq.heapify(num_az_q)
+        chosen_agents = []
+        while num_agents_needed > 0:
+            num, select_az = heapq.heappop(num_az_q)
+            select_agent = super(AZAwareWeightScheduler, self).select(
+                plugin, context, hostable_az_agents[select_az], [], 1)
+            chosen_agents.append(select_agent[0])
+            hostable_az_agents[select_az].remove(select_agent[0])
+            if hostable_az_agents[select_az]:
+                heapq.heappush(num_az_q, (num + 1, select_az))
+            num_agents_needed -= 1
+        return chosen_agents
+
+
 class DhcpFilter(base_resource_filter.BaseResourceFilter):
 
     def bind(self, context, agents, network_id):
@@ -115,11 +166,20 @@ class DhcpFilter(base_resource_filter.BaseResourceFilter):
         super(DhcpFilter, self).bind(context, bound_agents, network_id)
 
     def filter_agents(self, plugin, context, network):
-        """Return the agents that can host the network."""
+        """Return the agents that can host the network.
+
+        This function returns a dictionary which has 3 keys.
+        n_agents: The number of agents should be scheduled. If n_agents=0,
+        all networks are already scheduled or no more agent can host the
+        network.
+        hostable_agents: A list of agents which can host the network.
+        hosted_agents: A list of agents which already hosts the network.
+        """
         agents_dict = self._get_network_hostable_dhcp_agents(
                                     plugin, context, network)
         if not agents_dict['hostable_agents'] or agents_dict['n_agents'] <= 0:
-            return {'n_agents': 0, 'hostable_agents': []}
+            return {'n_agents': 0, 'hostable_agents': [],
+                    'hosted_agents': agents_dict['hosted_agents']}
         return agents_dict
 
     def _get_dhcp_agents_hosting_network(self, plugin, context, network):
@@ -138,30 +198,38 @@ class DhcpFilter(base_resource_filter.BaseResourceFilter):
                 return
         return network_hosted_agents
 
-    def _get_active_agents(self, plugin, context):
+    def _get_active_agents(self, plugin, context, az_hints):
         """Return a list of active dhcp agents."""
         with context.session.begin(subtransactions=True):
+            filters = {'agent_type': [constants.AGENT_TYPE_DHCP],
+                       'admin_state_up': [True]}
+            if az_hints:
+                filters['availability_zone'] = az_hints
             active_dhcp_agents = plugin.get_agents_db(
-                context, filters={
-                    'agent_type': [constants.AGENT_TYPE_DHCP],
-                    'admin_state_up': [True]})
+                context, filters=filters)
             if not active_dhcp_agents:
-                LOG.warn(_LW('No more DHCP agents'))
+                LOG.warning(_LW('No more DHCP agents'))
                 return []
         return active_dhcp_agents
 
     def _get_network_hostable_dhcp_agents(self, plugin, context, network):
-        """Return number of agents which will actually host the given network
-           and a list of dhcp agents which can host the given network
+        """Provide information on hostable DHCP agents for network.
+
+        The returned value includes the number of agents that will actually
+        host the given network, a list of DHCP agents that can host the given
+        network, and a list of DHCP agents currently hosting the network.
         """
         hosted_agents = self._get_dhcp_agents_hosting_network(plugin,
                                                               context, network)
         if hosted_agents is None:
-            return {'n_agents': 0, 'hostable_agents': []}
+            return {'n_agents': 0, 'hostable_agents': [], 'hosted_agents': []}
         n_agents = cfg.CONF.dhcp_agents_per_network - len(hosted_agents)
-        active_dhcp_agents = self._get_active_agents(plugin, context)
+        az_hints = (network.get(az_ext.AZ_HINTS) or
+                    cfg.CONF.default_availability_zones)
+        active_dhcp_agents = self._get_active_agents(plugin, context, az_hints)
         if not active_dhcp_agents:
-            return {'n_agents': 0, 'hostable_agents': []}
+            return {'n_agents': 0, 'hostable_agents': [],
+                    'hosted_agents': hosted_agents}
         hostable_dhcp_agents = [
             agent for agent in set(active_dhcp_agents)
             if agent not in hosted_agents and plugin.is_eligible_agent(
@@ -169,7 +237,8 @@ class DhcpFilter(base_resource_filter.BaseResourceFilter):
         ]
 
         if not hostable_dhcp_agents:
-            return {'n_agents': 0, 'hostable_agents': []}
+            return {'n_agents': 0, 'hostable_agents': [],
+                    'hosted_agents': hosted_agents}
         n_agents = min(len(hostable_dhcp_agents), n_agents)
-        return {'n_agents': n_agents, 'hostable_agents':
-                hostable_dhcp_agents}
+        return {'n_agents': n_agents, 'hostable_agents': hostable_dhcp_agents,
+                'hosted_agents': hosted_agents}
