@@ -12,20 +12,22 @@
 #    under the License.
 
 import eventlet
-
 from oslo_log import log
 
-from neutron.i18n import _LI
-from neutron import server
+from neutron._i18n import _LI
 from neutron import service
 
 LOG = log.getLogger(__name__)
 
 
-def _eventlet_wsgi_server():
+def eventlet_wsgi_server():
+    neutron_api = service.serve_wsgi(service.NeutronApiService)
+    start_api_and_rpc_workers(neutron_api)
+
+
+def start_api_and_rpc_workers(neutron_api):
     pool = eventlet.GreenPool()
 
-    neutron_api = service.serve_wsgi(service.NeutronApiService)
     api_thread = pool.spawn(neutron_api.wait)
 
     try:
@@ -45,7 +47,3 @@ def _eventlet_wsgi_server():
         api_thread.link(lambda gt: rpc_thread.kill())
 
     pool.waitall()
-
-
-def main():
-    server.boot_server(_eventlet_wsgi_server)
