@@ -10,8 +10,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from tempest import test
+
 from neutron.tests.api import base
-from neutron.tests.tempest import test
 
 
 class ServiceTypeManagementTest(base.BaseNetworkTest):
