@@ -66,10 +66,6 @@ def fake_use_fatal_exceptions(*args):
     return True
 
 
-def fake_consume_in_threads(self):
-    return []
-
-
 def get_rand_name(max_length=None, prefix='test'):
     """Return a random string.
 
@@ -318,9 +314,9 @@ class BaseTestCase(DietTestCase):
 
     def setup_rpc_mocks(self):
         # don't actually start RPC listeners when testing
-        self.useFixture(fixtures.MonkeyPatch(
+        mock.patch(
             'neutron.common.rpc.Connection.consume_in_threads',
-            fake_consume_in_threads))
+            return_value=[]).start()
 
         self.useFixture(fixtures.MonkeyPatch(
             'oslo_messaging.Notifier', fake_notifier.FakeNotifier))
@@ -428,3 +424,34 @@ class PluginFixture(fixtures.Fixture):
             if plugin() and not isinstance(plugin(), mock.Base):
                 raise AssertionError(
                     'The plugin for this test was not deallocated.')
+
+
+class Timeout(fixtures.Fixture):
+    """Setup per test timeouts.
+
+    In order to avoid test deadlocks we support setting up a test
+    timeout parameter read from the environment. In almost all
+    cases where the timeout is reached this means a deadlock.
+
+    A scaling factor allows extremely long tests to specify they
+    need more time.
+    """
+
+    def __init__(self, timeout=None, scaling=1):
+        super(Timeout, self).__init__()
+        if timeout is None:
+            timeout = os.environ.get('OS_TEST_TIMEOUT', 0)
+        try:
+            self.test_timeout = int(timeout)
+        except ValueError:
+            # If timeout value is invalid do not set a timeout.
+            self.test_timeout = 0
+        if scaling >= 1:
+            self.test_timeout *= scaling
+        else:
+            raise ValueError('scaling value must be >= 1')
+
+    def setUp(self):
+        super(Timeout, self).setUp()
+        if self.test_timeout > 0:
+            self.useFixture(fixtures.Timeout(self.test_timeout, gentle=True))
