@@ -15,6 +15,7 @@
 import hashlib
 import sys
 
+from oslo_utils import encodeutils
 
 from neutron.cmd.eventlet.plugins.ovs_neutron_agent import main as _main
 from neutron.common import constants as n_const
@@ -24,16 +25,20 @@ from neutron.plugins.ml2.drivers.openvswitch.agent.ovs_neutron_agent \
 
 def get_tunnel_name_full(cls, network_type, local_ip, remote_ip):
     network_type = network_type[:3]
-    remote_ip_hex = cls.get_ip_in_hex(remote_ip)
-    if not remote_ip_hex:
-        return None
-
     # Remove length of network_type and two dashes
     hashlen = (n_const.DEVICE_NAME_MAX_LEN - len(network_type) - 2) // 2
-    remote_ip_hex = remote_ip_hex.encode('utf-8')
-    remote_ip_hash = hashlib.sha1(remote_ip_hex).hexdigest()[0:hashlen]
-    local_ip_hex = cls.get_ip_in_hex(local_ip).encode('utf-8')
-    source_ip_hash = hashlib.sha1(local_ip_hex).hexdigest()[0:hashlen]
+
+    remote_tunnel_hash = cls.get_tunnel_hash(remote_ip, hashlen)
+    if not remote_tunnel_hash:
+        return None
+
+    remote_tunnel_hash = encodeutils.to_utf8(remote_tunnel_hash)
+    remote_ip_hash = hashlib.sha1(remote_tunnel_hash).hexdigest()[:hashlen]
+
+    local_tunnel_hash = cls.get_tunnel_hash(local_ip, hashlen)
+    local_tunnel_hash = encodeutils.to_utf8(local_tunnel_hash)
+    source_ip_hash = hashlib.sha1(local_tunnel_hash).hexdigest()[:hashlen]
+
     return '%s-%s-%s' % (network_type, source_ip_hash, remote_ip_hash)
 
 OVSNeutronAgent.get_tunnel_name = get_tunnel_name_full
