@@ -1,4 +1,4 @@
-# Copyright (c) 2015 Openstack Foundation
+# Copyright (c) 2015 OpenStack Foundation
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -13,6 +13,7 @@
 #    under the License.
 
 import copy
+
 import netaddr
 from oslo_utils import uuidutils
 from six import moves
@@ -31,7 +32,7 @@ def get_ha_interface(ip='169.254.192.1', mac='12:34:56:78:2b:5d'):
     subnet_id = _uuid()
     return {'admin_state_up': True,
             'device_id': _uuid(),
-            'device_owner': 'network:router_ha_interface',
+            'device_owner': l3_constants.DEVICE_OWNER_ROUTER_HA_INTF,
             'fixed_ips': [{'ip_address': ip,
                            'prefixlen': 18,
                            'subnet_id': subnet_id}],
@@ -52,7 +53,7 @@ def get_ha_interface(ip='169.254.192.1', mac='12:34:56:78:2b:5d'):
 def prepare_router_data(ip_version=4, enable_snat=None, num_internal_ports=1,
                         enable_floating_ip=False, enable_ha=False,
                         extra_routes=False, dual_stack=False,
-                        v6_ext_gw_with_sub=True, **kwargs):
+                        enable_gw=True, v6_ext_gw_with_sub=True, **kwargs):
     fixed_ips = []
     subnets = []
     gateway_mac = kwargs.get('gateway_mac', 'ca:fe:de:ad:be:ee')
@@ -85,12 +86,14 @@ def prepare_router_data(ip_version=4, enable_snat=None, num_internal_ports=1,
         raise ValueError("Invalid ip_version: %s" % ip_version)
 
     router_id = _uuid()
-    ex_gw_port = {'id': _uuid(),
-                  'mac_address': gateway_mac,
-                  'network_id': _uuid(),
-                  'fixed_ips': fixed_ips,
-                  'subnets': subnets,
-                  'extra_subnets': extra_subnets}
+    ex_gw_port = {}
+    if enable_gw:
+        ex_gw_port = {'id': _uuid(),
+                      'mac_address': gateway_mac,
+                      'network_id': _uuid(),
+                      'fixed_ips': fixed_ips,
+                      'subnets': subnets,
+                      'extra_subnets': extra_subnets}
 
     routes = []
     if extra_routes:
@@ -176,7 +179,7 @@ def router_append_interface(router, count=1, ip_version=4, ra_mode=None,
 
 def router_append_subnet(router, count=1, ip_version=4,
                          ipv6_subnet_modes=None, interface_id=None,
-                         network_mtu=0):
+                         dns_nameservers=None, network_mtu=0):
     if ip_version == 6:
         subnet_mode_none = {'ra_mode': None, 'address_mode': None}
         if not ipv6_subnet_modes:
@@ -223,6 +226,7 @@ def router_append_subnet(router, count=1, ip_version=4,
                 {'id': subnet_id,
                  'cidr': cidr_pool % (i + num_existing_subnets),
                  'gateway_ip': gw_pool % (i + num_existing_subnets),
+                 'dns_nameservers': dns_nameservers,
                  'ipv6_ra_mode': ipv6_subnet_modes[i]['ra_mode'],
                  'ipv6_address_mode': ipv6_subnet_modes[i]['address_mode']})
 
