@@ -17,6 +17,7 @@ import functools
 import fixtures
 from neutronclient.common import exceptions
 
+from neutron.extensions import portbindings
 from neutron.tests import base
 
 
@@ -81,7 +82,7 @@ class ClientFixture(fixtures.Fixture):
         spec = {
             'network_id': network_id,
             'tenant_id': tenant_id,
-            'binding:host_id': hostname,
+            portbindings.HOST_ID: hostname,
         }
         if qos_policy_id:
             spec['qos_policy_id'] = qos_policy_id
