@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import copy
 import functools
 
 import mock
@@ -27,6 +28,7 @@ from neutron.agent.linux import ip_lib
 from neutron.agent.linux import iptables_manager
 from neutron.agent.linux import utils
 from neutron.common import constants as l3_constants
+from neutron.common import exceptions as n_exc
 from neutron.extensions import portbindings
 from neutron.tests.common import l3_test_common
 from neutron.tests.common import machine_fixtures
@@ -112,6 +114,89 @@ class TestDvrRouter(framework.L3AgentTestFramework):
         self.assertEqual(3, default_rules_list_count)
         self.assertEqual(2, interface_rules_list_count)
 
+    def test_dvr_update_gateway_port_with_no_gw_port_in_namespace(self):
+        self.agent.conf.agent_mode = 'dvr'
+
+        # Create the router with external net
+        router_info = self.generate_dvr_router_info()
+        external_gw_port = router_info['gw_port']
+        ext_net_id = router_info['_floatingips'][0]['floating_network_id']
+        self.mock_plugin_api.get_external_network_id.return_value = ext_net_id
+        router = self.manage_router(self.agent, router_info)
+        fg_port = router.fip_ns.agent_gateway_port
+        fg_port_name = router.fip_ns.get_ext_device_name(fg_port['id'])
+        fg_device = ip_lib.IPDevice(fg_port_name,
+                                    namespace=router.fip_ns.name)
+        # Now validate if the gateway is properly configured.
+        self.assertIn('gateway', fg_device.route.get_gateway())
+        self._validate_fips_for_external_network(
+            router, router.fip_ns.get_name())
+        # Now delete the fg- port that was created
+        ext_net_bridge = self.agent.conf.external_network_bridge
+        router.fip_ns.driver.unplug(fg_port_name,
+                                    bridge=ext_net_bridge,
+                                    namespace=router.fip_ns.name,
+                                    prefix=dvr_fip_ns.FIP_EXT_DEV_PREFIX)
+        # Now check if the fg- port is missing.
+        self.assertFalse(fg_device.exists())
+        # Now change the gateway ip for the router and do an update.
+        router.ex_gw_port = copy.deepcopy(router.ex_gw_port)
+        new_fg_port = copy.deepcopy(fg_port)
+        for subnet in new_fg_port['subnets']:
+            subnet['gateway_ip'] = '19.4.4.2'
+        router.router[l3_constants.FLOATINGIP_AGENT_INTF_KEY] = [new_fg_port]
+        self.assertRaises(n_exc.FloatingIpSetupException,
+                          self.manage_router,
+                          self.agent,
+                          router.router)
+        router = self.manage_router(self.agent, router.router)
+        self.assertTrue(fg_device.exists())
+        self.assertEqual({'gateway': u'19.4.4.2'},
+                         fg_device.route.get_gateway())
+        self._validate_fips_for_external_network(
+            router, router.fip_ns.get_name())
+        self._delete_router(self.agent, router.router_id)
+        self._assert_fip_namespace_deleted(external_gw_port)
+
+    @mock.patch.object(dvr_fip_ns.FipNamespace, 'subscribe')
+    def test_dvr_process_fips_with_no_gw_port_in_namespace(
+        self, fip_subscribe):
+        self.agent.conf.agent_mode = 'dvr'
+
+        # Create the router with external net
+        router_info = self.generate_dvr_router_info()
+        external_gw_port = router_info['gw_port']
+        ext_net_id = router_info['_floatingips'][0]['floating_network_id']
+        self.mock_plugin_api.get_external_network_id.return_value = ext_net_id
+
+        # Create the fip namespace up front
+        dvr_fip_ns.FipNamespace(ext_net_id,
+                                self.agent.conf,
+                                self.agent.driver,
+                                self.agent.use_ipv6).create()
+        # Create the router with the fip, this shouldn't allow the
+        # update_gateway_port to be called without the fg- port
+        fip_subscribe.return_value = False
+        # This will raise the exception and will also clear
+        # subscription for the ext_net_id
+        self.assertRaises(n_exc.FloatingIpSetupException,
+                          self.manage_router,
+                          self.agent,
+                          router_info)
+        fip_subscribe.return_value = True
+        # Now update the router again
+        router = self.manage_router(self.agent, router_info)
+        fg_port = router.fip_ns.agent_gateway_port
+        fg_port_name = router.fip_ns.get_ext_device_name(fg_port['id'])
+        fg_device = ip_lib.IPDevice(fg_port_name,
+                                    namespace=router.fip_ns.name)
+        # Now validate if the gateway is properly configured.
+        self.assertIn('gateway', fg_device.route.get_gateway())
+        self._validate_fips_for_external_network(
+            router, router.fip_ns.get_name())
+        self._delete_router(self.agent, router.router_id)
+        self._assert_fip_namespace_deleted(external_gw_port)
+
     def test_dvr_router_fips_stale_gw_port(self):
         self.agent.conf.agent_mode = 'dvr'
 
@@ -150,7 +235,7 @@ class TestDvrRouter(framework.L3AgentTestFramework):
                            'prefixlen': prefixlen}],
             'id': framework._uuid(),
             'device_id': framework._uuid()}
-        stale_fip_ns.create_gateway_port(stale_agent_gw_port)
+        stale_fip_ns.create_or_update_gateway_port(stale_agent_gw_port)
 
         stale_dev_exists = self.device_exists_with_ips_and_mac(
                 stale_agent_gw_port,
@@ -940,6 +1025,34 @@ class TestDvrRouter(framework.L3AgentTestFramework):
         self._assert_extra_routes(router_updated, namespace=snat_ns_name)
         self._assert_extra_routes(router_updated)
 
+    def test_dvr_router_gateway_update_to_none(self):
+        self.agent.conf.agent_mode = 'dvr_snat'
+        router_info = self.generate_dvr_router_info(enable_snat=True)
+        router = self.manage_router(self.agent, router_info)
+        gw_port = router.get_ex_gw_port()
+        ex_gw_port_name = router.get_external_device_name(gw_port['id'])
+        ex_gw_device = ip_lib.IPDevice(ex_gw_port_name,
+                                       namespace=router.snat_namespace.name)
+        fg_port = router.fip_ns.agent_gateway_port
+        fg_port_name = router.fip_ns.get_ext_device_name(fg_port['id'])
+        fg_device = ip_lib.IPDevice(fg_port_name,
+                                    namespace=router.fip_ns.name)
+        self.assertIn('gateway', ex_gw_device.route.get_gateway())
+        self.assertIn('gateway', fg_device.route.get_gateway())
+
+        # Make this copy to make agent think gw_port changed.
+        router.ex_gw_port = copy.deepcopy(router.ex_gw_port)
+        for subnet in gw_port['subnets']:
+            subnet['gateway_ip'] = None
+        new_fg_port = copy.deepcopy(fg_port)
+        for subnet in new_fg_port['subnets']:
+            subnet['gateway_ip'] = None
+
+        router.router[l3_constants.FLOATINGIP_AGENT_INTF_KEY] = [new_fg_port]
+        router.process(self.agent)
+        self.assertIsNone(ex_gw_device.route.get_gateway())
+        self.assertIsNone(fg_device.route.get_gateway())
+
     def _assert_fip_namespace_deleted(self, ext_gateway_port):
         ext_net_id = ext_gateway_port['network_id']
         fip_ns = self.agent.get_fip_ns(ext_net_id)
@@ -1057,3 +1170,20 @@ class TestDvrRouter(framework.L3AgentTestFramework):
         # external networks. SNAT will be used. Direct route will not work
         # here.
         src_machine.assert_no_ping(machine_diff_scope.ip)
+
+    def test_dvr_snat_namespace_has_ip_nonlocal_bind_disabled(self):
+        self.agent.conf.agent_mode = 'dvr_snat'
+        router_info = self.generate_dvr_router_info(
+            enable_ha=True, enable_snat=True)
+        router = self.manage_router(self.agent, router_info)
+        try:
+            ip_nonlocal_bind_value = ip_lib.get_ip_nonlocal_bind(
+                router.snat_namespace.name)
+        except RuntimeError as rte:
+            stat_message = 'cannot stat /proc/sys/net/ipv4/ip_nonlocal_bind'
+            if stat_message in str(rte):
+                raise self.skipException(
+                    "This kernel doesn't support %s in network namespaces." % (
+                        ip_lib.IP_NONLOCAL_BIND))
+            raise
+        self.assertEqual(0, ip_nonlocal_bind_value)
