@@ -77,7 +77,9 @@ class L3HATestCase(framework.L3AgentTestFramework):
                 {router1.router_id: 'standby', router2.router_id: 'active'}))
 
     def test_ha_router_lifecycle(self):
-        self._router_lifecycle(enable_ha=True)
+        router_info = self._router_lifecycle(enable_ha=True)
+        # ensure everything was cleaned up
+        self._router_lifecycle(enable_ha=True, router_info=router_info)
 
     def test_conntrack_disassociate_fip_ha_router(self):
         self._test_conntrack_disassociate_fip(ha=True)
@@ -256,6 +258,33 @@ class L3HATestCase(framework.L3AgentTestFramework):
         # call the configure_fip_addresses directly here
         router.configure_fip_addresses(interface_name)
 
+    def test_ha_port_status_update(self):
+        router_info = self.generate_router_info(enable_ha=True)
+        router_info[l3_constants.HA_INTERFACE_KEY]['status'] = (
+            l3_constants.PORT_STATUS_DOWN)
+        router1 = self.manage_router(self.agent, router_info)
+        utils.wait_until_true(lambda: router1.ha_state == 'backup')
+
+        router1.router[l3_constants.HA_INTERFACE_KEY]['status'] = (
+            l3_constants.PORT_STATUS_ACTIVE)
+        self.agent._process_updated_router(router1.router)
+        utils.wait_until_true(lambda: router1.ha_state == 'master')
+
+    def test_ha_router_namespace_has_ip_nonlocal_bind_disabled(self):
+        router_info = self.generate_router_info(enable_ha=True)
+        router = self.manage_router(self.agent, router_info)
+        try:
+            ip_nonlocal_bind_value = ip_lib.get_ip_nonlocal_bind(
+                router.router_namespace.name)
+        except RuntimeError as rte:
+            stat_message = 'cannot stat /proc/sys/net/ipv4/ip_nonlocal_bind'
+            if stat_message in str(rte):
+                raise self.skipException(
+                    "This kernel doesn't support %s in network namespaces." % (
+                        ip_lib.IP_NONLOCAL_BIND))
+            raise
+        self.assertEqual(0, ip_nonlocal_bind_value)
+
 
 class L3HATestFailover(framework.L3AgentTestFramework):
 
@@ -300,3 +329,7 @@ class L3HATestFailover(framework.L3AgentTestFramework):
 
         utils.wait_until_true(lambda: router2.ha_state == 'master')
         utils.wait_until_true(lambda: router1.ha_state == 'backup')
+
+
+class LinuxBridgeL3HATestCase(L3HATestCase):
+    INTERFACE_DRIVER = 'neutron.agent.linux.interface.BridgeInterfaceDriver'
