@@ -14,6 +14,7 @@
 #    under the License.
 
 from neutron.common import constants
+from neutron.common import utils
 from neutron.plugins.ml2.drivers.linuxbridge.agent import arp_protect
 from neutron.tests.common import machine_fixtures
 from neutron.tests.common import net_helpers
@@ -34,10 +35,17 @@ class LinuxBridgeARPSpoofTestCase(functional_base.BaseSudoTestCase):
         bridge = lbfixture.bridge
         self.source, self.destination, self.observer = self.useFixture(
             machine_fixtures.PeerMachines(bridge, amount=3)).machines
+        self.addCleanup(self._ensure_rules_cleaned)
+
+    def _ensure_rules_cleaned(self):
+        rules = [r for r in arp_protect.ebtables(['-L']).splitlines()
+                 if r and 'Bridge' not in r]
+        self.assertEqual([], rules, 'Test leaked ebtables rules')
 
     def _add_arp_protection(self, machine, addresses, extra_port_dict=None):
         port_dict = {'fixed_ips': [{'ip_address': a} for a in addresses],
-                     'device_owner': 'nobody'}
+                     'device_owner': 'nobody',
+                     'mac_address': machine.port.link.address}
         if extra_port_dict:
             port_dict.update(extra_port_dict)
         name = net_helpers.VethFixture.get_peer_name(machine.port.name)
@@ -55,12 +63,38 @@ class LinuxBridgeARPSpoofTestCase(functional_base.BaseSudoTestCase):
         arping(self.source.namespace, self.destination.ip)
         arping(self.destination.namespace, self.source.ip)
 
+    def test_arp_correct_protection_allowed_address_pairs(self):
+        smac = self.source.port.link.address
+        port = {'mac_address': '00:11:22:33:44:55',
+                'allowed_address_pairs': [{'mac_address': smac,
+                                           'ip_address': self.source.ip}]}
+        # make sure a large number of allowed address pairs works
+        for i in range(100000):
+            port['allowed_address_pairs'].append(
+                {'mac_address': utils.get_random_mac(
+                     'fa:16:3e:00:00:00'.split(':')),
+                 'ip_address': '10.10.10.10'})
+        self._add_arp_protection(self.source, ['1.2.2.2'], port)
+        self._add_arp_protection(self.destination, [self.destination.ip])
+        arping(self.source.namespace, self.destination.ip)
+        arping(self.destination.namespace, self.source.ip)
+
     def test_arp_fails_incorrect_protection(self):
         self._add_arp_protection(self.source, ['1.1.1.1'])
         self._add_arp_protection(self.destination, ['2.2.2.2'])
         no_arping(self.source.namespace, self.destination.ip)
         no_arping(self.destination.namespace, self.source.ip)
 
+    def test_arp_fails_incorrect_mac_protection(self):
+        # a bad mac filter on the source will prevent any traffic from it
+        self._add_arp_protection(self.source, [self.source.ip],
+                                 {'mac_address': '00:11:22:33:44:55'})
+        no_arping(self.source.namespace, self.destination.ip)
+        no_arping(self.destination.namespace, self.source.ip)
+        # correcting it should make it work
+        self._add_arp_protection(self.source, [self.source.ip])
+        arping(self.source.namespace, self.destination.ip)
+
     def test_arp_protection_removal(self):
         self._add_arp_protection(self.source, ['1.1.1.1'])
         self._add_arp_protection(self.destination, ['2.2.2.2'])
