@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import copy
 import os.path
 
 import eventlet
@@ -83,10 +84,11 @@ class DHCPAgentOVSTestFramework(base.BaseSudoTestCase):
 
         self.conf.set_override('check_child_processes_interval', 1, 'AGENT')
 
-    def network_dict_for_dhcp(self, dhcp_enabled=True, ip_version=4):
+    def network_dict_for_dhcp(self, dhcp_enabled=True, ip_version=4,
+                              prefix_override=None):
         net_id = uuidutils.generate_uuid()
         subnet_dict = self.create_subnet_dict(
-            net_id, dhcp_enabled, ip_version)
+            net_id, dhcp_enabled, ip_version, prefix_override)
         port_dict = self.create_port_dict(
             net_id, subnet_dict.id,
             mac_address=str(self._DHCP_PORT_MAC_ADDRESS),
@@ -97,12 +99,16 @@ class DHCPAgentOVSTestFramework(base.BaseSudoTestCase):
             net_id, [subnet_dict], [port_dict])
         return net_dict
 
-    def create_subnet_dict(self, net_id, dhcp_enabled=True, ip_version=4):
+    def create_subnet_dict(self, net_id, dhcp_enabled=True, ip_version=4,
+                           prefix_override=None):
+        cidr = self._IP_ADDRS[ip_version]['cidr']
+        if prefix_override is not None:
+            cidr = '/'.join((cidr.split('/')[0], str(prefix_override)))
         sn_dict = dhcp.DictModel({
             "id": uuidutils.generate_uuid(),
             "network_id": net_id,
             "ip_version": ip_version,
-            "cidr": self._IP_ADDRS[ip_version]['cidr'],
+            "cidr": cidr,
             "gateway_ip": (self.
                 _IP_ADDRS[ip_version]['gateway']),
             "enable_dhcp": dhcp_enabled,
@@ -253,6 +259,18 @@ class DHCPAgentOVSTestCase(DHCPAgentOVSTestFramework):
                                             dhcp_enabled=dhcp_enabled)
             self.assert_dhcp_resources(network, dhcp_enabled)
 
+    def test_create_subnet_with_non64_ipv6_cidrs(self):
+        # the agent should not throw exceptions on weird prefixes
+        dhcp_enabled = True
+        version = 6
+        for i in (0, 1, 41, 81, 121, 127, 128):
+            network = self.network_dict_for_dhcp(
+                dhcp_enabled, ip_version=version, prefix_override=i)
+            self.configure_dhcp_for_network(network=network,
+                                            dhcp_enabled=dhcp_enabled)
+            self.assertFalse(self.agent.needs_resync_reasons[network.id],
+                             msg="prefix size of %s triggered resync" % i)
+
     def test_agent_mtu_set_on_interface_driver(self):
         network = self.network_dict_for_dhcp()
         network["mtu"] = 789
@@ -313,3 +331,42 @@ class DHCPAgentOVSTestCase(DHCPAgentOVSTestFramework):
             timeout=5,
             sleep=0.1,
             exception=RuntimeError("Stale metadata proxy didn't get killed"))
+
+    def _test_metadata_proxy_spawn_kill_with_subnet_create_delete(self):
+        network = self.network_dict_for_dhcp(ip_version=6)
+        self.configure_dhcp_for_network(network=network)
+        pm = self._get_metadata_proxy_process(network)
+
+        # A newly created network with ipv6 subnet will not have metadata proxy
+        self.assertFalse(pm.active)
+
+        new_network = copy.deepcopy(network)
+        dhcp_enabled_ipv4_subnet = self.create_subnet_dict(network.id)
+        new_network.subnets.append(dhcp_enabled_ipv4_subnet)
+        self.mock_plugin_api.get_network_info.return_value = new_network
+        self.agent.refresh_dhcp_helper(network.id)
+        # Metadata proxy should be spawned for the newly added subnet
+        utils.wait_until_true(
+            lambda: pm.active,
+            timeout=5,
+            sleep=0.1,
+            exception=RuntimeError("Metadata proxy didn't spawn"))
+
+        self.mock_plugin_api.get_network_info.return_value = network
+        self.agent.refresh_dhcp_helper(network.id)
+        # Metadata proxy should be killed because network doesn't need it.
+        utils.wait_until_true(
+            lambda: not pm.active,
+            timeout=5,
+            sleep=0.1,
+            exception=RuntimeError("Metadata proxy didn't get killed"))
+
+    def test_enable_isolated_metadata_for_subnet_create_delete(self):
+        self.conf.set_override('force_metadata', False)
+        self.conf.set_override('enable_isolated_metadata', True)
+        self._test_metadata_proxy_spawn_kill_with_subnet_create_delete()
+
+    def test_force_metadata_for_subnet_create_delete(self):
+        self.conf.set_override('force_metadata', True)
+        self.conf.set_override('enable_isolated_metadata', False)
+        self._test_metadata_proxy_spawn_kill_with_subnet_create_delete()
