@@ -548,6 +548,147 @@ class L3DvrTestCase(ml2_test_base.ML2TestFramework):
                 self.assertEqual(
                     0, l3_notifier.routers_updated_on_host.call_count)
 
+    def test_allowed_addr_pairs_delayed_fip_and_update_arp_entry(self):
+        HOST1 = 'host1'
+        helpers.register_l3_agent(
+            host=HOST1, agent_mode=constants.L3_AGENT_MODE_DVR)
+        HOST2 = 'host2'
+        helpers.register_l3_agent(
+            host=HOST2, agent_mode=constants.L3_AGENT_MODE_DVR)
+        router = self._create_router()
+        private_net1 = self._make_network(self.fmt, 'net1', True)
+        test_allocation_pools = [{'start': '10.1.0.2',
+                                  'end': '10.1.0.20'}]
+        fixed_vrrp_ip = [{'ip_address': '10.1.0.201'}]
+        kwargs = {'arg_list': (external_net.EXTERNAL,),
+                  external_net.EXTERNAL: True}
+        ext_net = self._make_network(self.fmt, '', True, **kwargs)
+        self._make_subnet(
+            self.fmt, ext_net, '10.20.0.1', '10.20.0.0/24',
+            ip_version=4, enable_dhcp=True)
+        # Set gateway to router
+        self.l3_plugin._update_router_gw_info(
+            self.context, router['id'],
+            {'network_id': ext_net['network']['id']})
+        private_subnet1 = self._make_subnet(
+            self.fmt,
+            private_net1,
+            '10.1.0.1',
+            cidr='10.1.0.0/24',
+            ip_version=4,
+            allocation_pools=test_allocation_pools,
+            enable_dhcp=True)
+        vrrp_port = self._make_port(
+            self.fmt,
+            private_net1['network']['id'],
+            fixed_ips=fixed_vrrp_ip)
+        allowed_address_pairs = [
+            {'ip_address': '10.1.0.201',
+             'mac_address': vrrp_port['port']['mac_address']}]
+        with self.port(
+                subnet=private_subnet1,
+                device_owner=DEVICE_OWNER_COMPUTE) as int_port,\
+                self.port(subnet=private_subnet1,
+                          device_owner=DEVICE_OWNER_COMPUTE) as int_port2:
+            self.l3_plugin.add_router_interface(
+                self.context, router['id'],
+                {'subnet_id': private_subnet1['subnet']['id']})
+            with mock.patch.object(self.l3_plugin,
+                                   '_l3_rpc_notifier') as l3_notifier:
+                vm_port = self.core_plugin.update_port(
+                    self.context, int_port['port']['id'],
+                    {'port': {portbindings.HOST_ID: HOST1}})
+                vm_port_mac = vm_port['mac_address']
+                vm_port_fixed_ips = vm_port['fixed_ips']
+                vm_port_subnet_id = vm_port_fixed_ips[0]['subnet_id']
+                vm_arp_table = {
+                    'ip_address': vm_port_fixed_ips[0]['ip_address'],
+                    'mac_address': vm_port_mac,
+                    'subnet_id': vm_port_subnet_id}
+                vm_port2 = self.core_plugin.update_port(
+                    self.context, int_port2['port']['id'],
+                    {'port': {portbindings.HOST_ID: HOST2}})
+                l3_notifier.reset_mock()
+                # Now update the VM port with the allowed_address_pair
+                self.core_plugin.update_port(
+                     self.context, vm_port['id'],
+                     {'port': {
+                         'allowed_address_pairs': allowed_address_pairs}})
+                self.core_plugin.update_port(
+                     self.context, vm_port2['id'],
+                     {'port': {
+                         'allowed_address_pairs': allowed_address_pairs}})
+                self.assertEqual(
+                    0, l3_notifier.routers_updated_on_host.call_count)
+                updated_vm_port1 = self.core_plugin.get_port(
+                    self.context, vm_port['id'])
+                updated_vm_port2 = self.core_plugin.get_port(
+                    self.context, vm_port2['id'])
+                self.assertEqual(4, l3_notifier.add_arp_entry.call_count)
+                expected_allowed_address_pairs = updated_vm_port1.get(
+                    'allowed_address_pairs')
+                self.assertEqual(expected_allowed_address_pairs,
+                                 allowed_address_pairs)
+                expected_allowed_address_pairs_2 = updated_vm_port2.get(
+                    'allowed_address_pairs')
+                self.assertEqual(expected_allowed_address_pairs_2,
+                                 allowed_address_pairs)
+                # Now the VRRP port is attached to the VM port. At this
+                # point, the VRRP port should not have inherited the
+                # port host bindings from the parent VM port.
+                cur_vrrp_port_db = self.core_plugin.get_port(
+                    self.context, vrrp_port['port']['id'])
+                self.assertNotEqual(
+                    cur_vrrp_port_db[portbindings.HOST_ID], HOST1)
+                self.assertNotEqual(
+                    cur_vrrp_port_db[portbindings.HOST_ID], HOST2)
+                # Before we try to associate a floatingip make sure that
+                # only one of the Service port associated with the
+                # allowed_address_pair port is active and the other one
+                # is DOWN
+                mod_vm_port2 = self.core_plugin.update_port(
+                    self.context, updated_vm_port2['id'],
+                    {'port': {
+                        'admin_state_up': False}})
+                self.assertFalse(mod_vm_port2['admin_state_up'])
+                # Next we can try to associate the floatingip to the
+                # VRRP port that is already attached to the VM port
+                l3_notifier.reset_mock()
+                floating_ip = {'floating_network_id': ext_net['network']['id'],
+                               'router_id': router['id'],
+                               'port_id': vrrp_port['port']['id'],
+                               'tenant_id': vrrp_port['port']['tenant_id']}
+                floating_ip = self.l3_plugin.create_floatingip(
+                    self.context, {'floatingip': floating_ip})
+                self.assertEqual(
+                    2, l3_notifier.routers_updated_on_host.call_count)
+                self.assertEqual(3, l3_notifier.add_arp_entry.call_count)
+
+                post_update_vrrp_port_db = self.core_plugin.get_port(
+                    self.context, vrrp_port['port']['id'])
+                vrrp_port_fixed_ips = post_update_vrrp_port_db['fixed_ips']
+                vrrp_port_subnet_id = vrrp_port_fixed_ips[0]['subnet_id']
+                vrrp_arp_table = {
+                    'ip_address': vrrp_port_fixed_ips[0]['ip_address'],
+                    'mac_address': vm_port_mac,
+                    'subnet_id': vrrp_port_subnet_id}
+                vrrp_arp_table1 = {
+                    'ip_address': vrrp_port_fixed_ips[0]['ip_address'],
+                    'mac_address': vrrp_port['port']['mac_address'],
+                    'subnet_id': vrrp_port_subnet_id}
+
+                self.assertEqual(
+                    post_update_vrrp_port_db[portbindings.HOST_ID], HOST1)
+                expected_calls = [
+                        mock.call(self.context,
+                                  router['id'], vrrp_arp_table1),
+                        mock.call(self.context,
+                                  router['id'], vm_arp_table),
+                        mock.call(self.context,
+                                  router['id'], vrrp_arp_table)]
+                l3_notifier.add_arp_entry.assert_has_calls(
+                        expected_calls)
+
     def test_allowed_address_pairs_update_arp_entry(self):
         HOST1 = 'host1'
         helpers.register_l3_agent(
@@ -831,6 +972,64 @@ class L3DvrTestCase(ml2_test_base.ML2TestFramework):
                 l3_notifier.router_removed_from_agent.assert_called_once_with(
                     mock.ANY, router['id'], HOST1)
 
+    def test_dvr_router_manual_rescheduling_removes_router(self):
+        router = self._create_router()
+        kwargs = {'arg_list': (external_net.EXTERNAL,),
+                  external_net.EXTERNAL: True}
+        with self.network(**kwargs) as ext_net,\
+                self.subnet(network=ext_net),\
+                self.subnet(cidr='20.0.0.0/24') as subnet,\
+                self.port(subnet=subnet):
+            self.l3_plugin._update_router_gw_info(
+                self.context, router['id'],
+                {'network_id': ext_net['network']['id']})
+            self.l3_plugin.add_router_interface(
+                self.context, router['id'],
+                {'subnet_id': subnet['subnet']['id']})
+            self.l3_plugin.schedule_router(self.context,
+                                           router['id'],
+                                           candidates=[self.l3_agent])
+            # Now the VM should be also scheduled on the node
+            notifier = self.l3_plugin.agent_notifiers[
+                constants.AGENT_TYPE_L3]
+            with mock.patch.object(
+                    notifier, 'router_removed_from_agent') as rtr_remove_mock:
+                self.l3_plugin.remove_router_from_l3_agent(
+                    self.context, self.l3_agent['id'], router['id'])
+                rtr_remove_mock.assert_called_once_with(
+                    self.context, router['id'], self.l3_agent['host'])
+
+    def test_dvr_router_manual_rescheduling_updates_router(self):
+        router = self._create_router()
+        kwargs = {'arg_list': (external_net.EXTERNAL,),
+                  external_net.EXTERNAL: True}
+        with self.network(**kwargs) as ext_net,\
+                self.subnet(network=ext_net),\
+                self.subnet(cidr='20.0.0.0/24') as subnet,\
+                self.port(subnet=subnet,
+                          device_owner=DEVICE_OWNER_COMPUTE) as port:
+            self.core_plugin.update_port(
+                self.context, port['port']['id'],
+                {'port': {'binding:host_id': self.l3_agent['host']}})
+            self.l3_plugin._update_router_gw_info(
+                self.context, router['id'],
+                {'network_id': ext_net['network']['id']})
+            self.l3_plugin.add_router_interface(
+                self.context, router['id'],
+                {'subnet_id': subnet['subnet']['id']})
+            self.l3_plugin.schedule_router(self.context,
+                                           router['id'],
+                                           candidates=[self.l3_agent])
+            # Now the VM should be also scheduled on the node
+            notifier = self.l3_plugin.agent_notifiers[
+                constants.AGENT_TYPE_L3]
+            with mock.patch.object(
+                    notifier, 'routers_updated_on_host') as rtr_update_mock:
+                self.l3_plugin.remove_router_from_l3_agent(
+                    self.context, self.l3_agent['id'], router['id'])
+                rtr_update_mock.assert_called_once_with(
+                    self.context, [router['id']], self.l3_agent['host'])
+
     def _test_router_remove_from_agent_on_vm_port_deletion(
             self, non_admin_port=False):
         # register l3 agent in dvr mode in addition to existing dvr_snat agent
