@@ -13,6 +13,7 @@
 #    under the License.
 
 import collections
+
 import mock
 from oslo_serialization import jsonutils
 from oslo_utils import uuidutils
@@ -324,7 +325,7 @@ class OVS_Lib_Test(base.BaseTestCase):
         run_ofctl = mock.patch.object(self.br, 'run_ofctl').start()
         run_ofctl.side_effect = ['']
         retflows = self.br.dump_flows_for_table(table)
-        self.assertEqual(None, retflows)
+        self.assertIsNone(retflows)
 
     def test_mod_flow_with_priority_set(self):
         params = {'in_port': '1',
@@ -341,6 +342,21 @@ class OVS_Lib_Test(base.BaseTestCase):
                           self.br.mod_flow,
                           **params)
 
+    def test_run_ofctl_retry_on_socket_error(self):
+        err = RuntimeError('failed to connect to socket')
+        self.execute.side_effect = [err] * 5
+        with mock.patch('time.sleep') as sleep:
+            self.br.run_ofctl('add-flows', [])
+        self.assertEqual(5, sleep.call_count)
+        self.assertEqual(6, self.execute.call_count)
+        # a regular exception fails right away
+        self.execute.side_effect = RuntimeError('garbage')
+        self.execute.reset_mock()
+        with mock.patch('time.sleep') as sleep:
+            self.br.run_ofctl('add-flows', [])
+        self.assertEqual(0, sleep.call_count)
+        self.assertEqual(1, self.execute.call_count)
+
     def test_add_tunnel_port(self):
         pname = "tap99"
         local_ip = "1.1.1.1"
@@ -890,15 +906,20 @@ class TestDeferredOVSBridge(base.BaseTestCase):
         with ovs_lib.DeferredOVSBridge(self.br) as deferred_br:
             self.assertRaises(AttributeError, getattr, deferred_br, 'failure')
 
+    def test_default_cookie(self):
+        self.br = ovs_lib.OVSBridge("br-tun")
+        uuid_stamp1 = self.br.default_cookie
+        self.assertEqual(uuid_stamp1, self.br.default_cookie)
+
     def test_cookie_passed_to_addmod(self):
         self.br = ovs_lib.OVSBridge("br-tun")
-        self.br.set_agent_uuid_stamp(1234)
+        stamp = str(self.br.default_cookie)
         expected_calls = [
             mock.call('add-flows', ['-'],
                       'hard_timeout=0,idle_timeout=0,priority=1,'
-                      'cookie=1234,actions=drop'),
+                      'cookie=' + stamp + ',actions=drop'),
             mock.call('mod-flows', ['-'],
-                      'cookie=1234,actions=drop')
+                      'cookie=' + stamp + ',actions=drop')
         ]
         with mock.patch.object(self.br, 'run_ofctl') as f:
             with ovs_lib.DeferredOVSBridge(self.br) as deferred_br:
