@@ -115,6 +115,7 @@ fake_port2 = dhcp.DictModel(dict(id='12345678-1234-aaaa-123456789000',
                             device_owner='',
                             mac_address='aa:bb:cc:dd:ee:99',
                             network_id='12345678-1234-5678-1234567890ab',
+                            revision_number=77,
                             fixed_ips=[fake_fixed_ip2]))
 
 fake_ipv6_port = dhcp.DictModel(dict(id='12345678-1234-aaaa-123456789000',
@@ -569,6 +570,7 @@ class TestDhcpAgentEventHandler(base.BaseTestCase):
         self.cache_p = mock.patch('neutron.agent.dhcp.agent.NetworkCache')
         cache_cls = self.cache_p.start()
         self.cache = mock.Mock()
+        self.cache.is_port_message_stale.return_value = False
         cache_cls.return_value = self.cache
         self.mock_makedirs_p = mock.patch("os.makedirs")
         self.mock_makedirs = self.mock_makedirs_p.start()
@@ -607,12 +609,10 @@ class TestDhcpAgentEventHandler(base.BaseTestCase):
             self.external_process.assert_has_calls([
                 self._process_manager_constructor_call(),
                 mock.call().enable()])
-        elif not enable_isolated_metadata:
+        else:
             self.external_process.assert_has_calls([
                 self._process_manager_constructor_call(ns=None),
                 mock.call().disable()])
-        else:
-            self.assertFalse(self.external_process.call_count)
 
     def test_enable_dhcp_helper_enable_metadata_isolated_network(self):
         self._enable_dhcp_helper(isolated_network,
@@ -1014,6 +1014,7 @@ class TestDhcpAgentEventHandler(base.BaseTestCase):
         self.dhcp.port_delete_end(None, payload)
         self.cache.assert_has_calls(
             [mock.call.get_port_by_id(fake_port2.id),
+             mock.call.deleted_ports.add(fake_port2.id),
              mock.call.get_network_by_id(fake_network.id),
              mock.call.remove_port(fake_port2)])
         self.call_driver.assert_has_calls(
@@ -1071,6 +1072,21 @@ class TestDhcpPluginApiProxy(base.BaseTestCase):
 
 
 class TestNetworkCache(base.BaseTestCase):
+
+    def test_update_of_deleted_port_ignored(self):
+        nc = dhcp_agent.NetworkCache()
+        nc.put(fake_network)
+        nc.deleted_ports.add(fake_port2['id'])
+        self.assertTrue(nc.is_port_message_stale(fake_port2))
+
+    def test_stale_update_ignored(self):
+        nc = dhcp_agent.NetworkCache()
+        nc.put(fake_network)
+        nc.put_port(fake_port2)
+        stale = copy.copy(fake_port2)
+        stale['revision_number'] = 2
+        self.assertTrue(nc.is_port_message_stale(stale))
+
     def test_put_network(self):
         nc = dhcp_agent.NetworkCache()
         nc.put(fake_network)
