@@ -22,6 +22,8 @@ from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants \
         as ovs_consts
 from neutron.tests import base
 
+TESTING_VLAN_TAG = 1
+
 
 class TestIsValidPrefix(base.BaseTestCase):
     def test_valid_prefix_ipv4(self):
@@ -51,7 +53,8 @@ class TestCreateFlowsFromRuleAndPort(base.BaseTestCase):
         ovs_port = mock.Mock(vif_mac='00:00:00:00:00:00')
         ovs_port.ofport = 1
         port_dict = {'device': 'port_id'}
-        self.port = ovsfw.OFPort(port_dict, ovs_port)
+        self.port = ovsfw.OFPort(
+            port_dict, ovs_port, vlan_tag=TESTING_VLAN_TAG)
 
         self.create_flows_mock = mock.patch.object(
             rules, 'create_protocol_flows').start()
@@ -74,7 +77,7 @@ class TestCreateFlowsFromRuleAndPort(base.BaseTestCase):
         expected_template = {
             'priority': 70,
             'dl_type': constants.ETHERTYPE_IP,
-            'reg5': self.port.ofport,
+            'reg_port': self.port.ofport,
         }
         self._test_create_flows_from_rule_and_port_helper(rule,
                                                           expected_template)
@@ -89,7 +92,7 @@ class TestCreateFlowsFromRuleAndPort(base.BaseTestCase):
         expected_template = {
             'priority': 70,
             'dl_type': constants.ETHERTYPE_IP,
-            'reg5': self.port.ofport,
+            'reg_port': self.port.ofport,
             'nw_src': '192.168.0.0/24',
             'nw_dst': '10.0.0.1/32',
         }
@@ -106,7 +109,7 @@ class TestCreateFlowsFromRuleAndPort(base.BaseTestCase):
         expected_template = {
             'priority': 70,
             'dl_type': constants.ETHERTYPE_IP,
-            'reg5': self.port.ofport,
+            'reg_port': self.port.ofport,
             'nw_src': '192.168.0.0/24',
         }
         self._test_create_flows_from_rule_and_port_helper(rule,
@@ -120,7 +123,7 @@ class TestCreateFlowsFromRuleAndPort(base.BaseTestCase):
         expected_template = {
             'priority': 70,
             'dl_type': constants.ETHERTYPE_IPV6,
-            'reg5': self.port.ofport,
+            'reg_port': self.port.ofport,
         }
         self._test_create_flows_from_rule_and_port_helper(rule,
                                                           expected_template)
@@ -135,7 +138,7 @@ class TestCreateFlowsFromRuleAndPort(base.BaseTestCase):
         expected_template = {
             'priority': 70,
             'dl_type': constants.ETHERTYPE_IPV6,
-            'reg5': self.port.ofport,
+            'reg_port': self.port.ofport,
             'ipv6_src': '2001:db8:bbbb::1/64',
             'ipv6_dst': '2001:db8:aaaa::1/64',
         }
@@ -152,7 +155,7 @@ class TestCreateFlowsFromRuleAndPort(base.BaseTestCase):
         expected_template = {
             'priority': 70,
             'dl_type': constants.ETHERTYPE_IPV6,
-            'reg5': self.port.ofport,
+            'reg_port': self.port.ofport,
             'ipv6_src': '2001:db8:bbbb::1/64',
         }
         self._test_create_flows_from_rule_and_port_helper(rule,
@@ -165,7 +168,8 @@ class TestCreateProtocolFlows(base.BaseTestCase):
         ovs_port = mock.Mock(vif_mac='00:00:00:00:00:00')
         ovs_port.ofport = 1
         port_dict = {'device': 'port_id'}
-        self.port = ovsfw.OFPort(port_dict, ovs_port)
+        self.port = ovsfw.OFPort(
+            port_dict, ovs_port, vlan_tag=TESTING_VLAN_TAG)
 
     def _test_create_protocol_flows_helper(self, direction, rule,
                                            expected_flows):
@@ -181,7 +185,7 @@ class TestCreateProtocolFlows(base.BaseTestCase):
         expected_flows = [{
             'table': ovs_consts.RULES_INGRESS_TABLE,
             'dl_dst': self.port.mac,
-            'actions': 'ct(commit,zone=NXM_NX_REG5[0..15]),output:1',
+            'actions': 'strip_vlan,output:1',
             'nw_proto': constants.PROTO_NUM_TCP,
         }]
         self._test_create_protocol_flows_helper(
