@@ -13,12 +13,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import copy
+
 import mock
 from oslo_versionedobjects import base as obj_base
 from oslo_versionedobjects import fields as obj_fields
 import testtools
 
 from neutron.api.rpc.callbacks import resources
+from neutron.api.rpc.callbacks import version_manager
 from neutron.api.rpc.handlers import resources_rpc
 from neutron.common import topics
 from neutron import context
@@ -38,8 +41,9 @@ def _create_test_resource(context=None):
     return resource
 
 
-@obj_base.VersionedObjectRegistry.register
 class FakeResource(objects_base.NeutronObject):
+    # Version 1.0: Initial version
+    VERSION = '1.0'
 
     fields = {
         'id': obj_fields.UUIDField(),
@@ -55,8 +59,22 @@ class ResourcesRpcBaseTestCase(base.BaseTestCase):
 
     def setUp(self):
         super(ResourcesRpcBaseTestCase, self).setUp()
+
+        # TODO(mhickey) This is using temp registry pattern. The
+        # pattern solution is to backup the object registry, register
+        # a class locally, and then restore the original registry.
+        # Refer to https://review.openstack.org/#/c/263800/ for more
+        # details. This code should be updated when the patch is merged.
+        self._base_test_backup = copy.copy(
+            obj_base.VersionedObjectRegistry._registry._obj_classes)
+        self.addCleanup(self._restore_obj_registry)
+
         self.context = context.get_admin_context()
 
+    def _restore_obj_registry(self):
+        obj_base.VersionedObjectRegistry._registry._obj_classes = (
+            self._base_test_backup)
+
 
 class _ValidateResourceTypeTestCase(base.BaseTestCase):
     def setUp(self):
@@ -103,6 +121,7 @@ class ResourcesPullRpcApiTestCase(ResourcesRpcBaseTestCase):
         self.assertIs(self.rpc, resources_rpc.ResourcesPullRpcApi())
 
     def test_pull(self):
+        obj_base.VersionedObjectRegistry.register(FakeResource)
         expected_obj = _create_test_resource(self.context)
         resource_id = expected_obj.id
         self.cctxt_mock.call.return_value = expected_obj.obj_to_primitive()
@@ -124,10 +143,26 @@ class ResourcesPullRpcApiTestCase(ResourcesRpcBaseTestCase):
                           resource_id)
 
 
+class ResourcesPushToServerRpcCallbackTestCase(ResourcesRpcBaseTestCase):
+
+    def test_report_versions(self):
+        callbacks = resources_rpc.ResourcesPushToServerRpcCallback()
+        with mock.patch('neutron.api.rpc.callbacks.version_manager'
+                        '.update_versions') as update_versions:
+            version_map = {'A': '1.0'}
+            callbacks.report_agent_resource_versions(context=mock.ANY,
+                                      agent_type='DHCP Agent',
+                                      agent_host='fake-host',
+                                      version_map=version_map)
+            update_versions.assert_called_once_with(mock.ANY,
+                                                    version_map)
+
+
 class ResourcesPullRpcCallbackTestCase(ResourcesRpcBaseTestCase):
 
     def setUp(self):
         super(ResourcesPullRpcCallbackTestCase, self).setUp()
+        obj_base.VersionedObjectRegistry.register(FakeResource)
         self.callbacks = resources_rpc.ResourcesPullRpcCallback()
         self.resource_obj = _create_test_resource(self.context)
 
@@ -175,7 +210,7 @@ class ResourcesPushRpcApiTestCase(ResourcesRpcBaseTestCase):
         with mock.patch.object(resources_rpc.resources, 'get_resource_cls',
                 return_value=FakeResource):
             observed = self.rpc._prepare_object_fanout_context(
-                self.resource_obj)
+                self.resource_obj, self.resource_obj.VERSION)
 
         self.rpc.client.prepare.assert_called_once_with(
             fanout=True, topic=expected_topic)
@@ -184,8 +219,10 @@ class ResourcesPushRpcApiTestCase(ResourcesRpcBaseTestCase):
     def test_pushy(self):
         with mock.patch.object(resources_rpc.resources, 'get_resource_cls',
                 return_value=FakeResource):
-            self.rpc.push(
-                self.context, self.resource_obj, 'TYPE')
+            with mock.patch.object(version_manager, 'get_resource_versions',
+                    return_value=set([FakeResource.VERSION])):
+                self.rpc.push(
+                    self.context, self.resource_obj, 'TYPE')
 
         self.cctxt_mock.cast.assert_called_once_with(
             self.context, 'push',
@@ -207,6 +244,7 @@ class ResourcesPushRpcCallbackTestCase(ResourcesRpcBaseTestCase):
 
     @mock.patch.object(resources_rpc.cons_registry, 'push')
     def test_push(self, reg_push_mock):
+        obj_base.VersionedObjectRegistry.register(FakeResource)
         self.callbacks.push(self.context, self.resource_prim, 'TYPE')
         reg_push_mock.assert_called_once_with(self.resource_obj.obj_name(),
                                               self.resource_obj, 'TYPE')
