@@ -19,6 +19,7 @@ import mock
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
+from oslo_service import wsgi as base_wsgi
 import routes
 import six
 import webob
@@ -48,9 +49,10 @@ _get_path = test_base._get_path
 extensions_path = ':'.join(neutron.tests.unit.extensions.__path__)
 
 
-class ExtensionsTestApp(wsgi.Router):
+class ExtensionsTestApp(base_wsgi.Router):
 
-    def __init__(self, options={}):
+    def __init__(self, options=None):
+        options = options or {}
         mapper = routes.Mapper()
         controller = ext_stubs.StubBaseAppController()
         mapper.resource("dummy_resource", "/dummy_resources",
@@ -80,7 +82,7 @@ class ExtensionPathTest(base.BaseTestCase):
                          '%s:neutron/tests/unit/extensions' % self.base_path)
 
     def test_get_extensions_path_no_extensions(self):
-        # Reset to default value, as it's overriden by base class
+        # Reset to default value, as it's overridden by base class
         cfg.CONF.set_override('api_extensions_path', '')
         path = extensions.get_extensions_path()
         self.assertEqual(path, self.base_path)
@@ -158,6 +160,9 @@ class ResourceExtensionTest(base.BaseTestCase):
         def custom_member_action(self, request, id):
             return {'member_action': 'value'}
 
+        def custom_collection_method(self, request, **kwargs):
+            return {'collection': 'value'}
+
         def custom_collection_action(self, request, **kwargs):
             return {'collection': 'value'}
 
@@ -353,6 +358,80 @@ class ResourceExtensionTest(base.BaseTestCase):
         self.assertEqual(200, response.status_int)
         self.assertEqual(jsonutils.loads(response.body)['collection'], "value")
 
+    def test_resource_extension_for_get_custom_collection_method(self):
+        controller = self.ResourceExtensionController()
+        collections = {'custom_collection_method': "GET"}
+        res_ext = extensions.ResourceExtension('tweedles', controller,
+                                               collection_methods=collections)
+        test_app = _setup_extensions_test_app(SimpleExtensionManager(res_ext))
+
+        response = test_app.get("/tweedles")
+
+        self.assertEqual(200, response.status_int)
+        self.assertEqual("value", jsonutils.loads(response.body)['collection'])
+
+    def test_resource_extension_for_put_custom_collection_method(self):
+        controller = self.ResourceExtensionController()
+        collections = {'custom_collection_method': "PUT"}
+        res_ext = extensions.ResourceExtension('tweedles', controller,
+                                               collection_methods=collections)
+        test_app = _setup_extensions_test_app(SimpleExtensionManager(res_ext))
+
+        response = test_app.put("/tweedles")
+
+        self.assertEqual(200, response.status_int)
+        self.assertEqual('value', jsonutils.loads(response.body)['collection'])
+
+    def test_resource_extension_for_post_custom_collection_method(self):
+        controller = self.ResourceExtensionController()
+        collections = {'custom_collection_method': "POST"}
+        res_ext = extensions.ResourceExtension('tweedles', controller,
+                                               collection_methods=collections)
+        test_app = _setup_extensions_test_app(SimpleExtensionManager(res_ext))
+
+        response = test_app.post("/tweedles")
+
+        self.assertEqual(200, response.status_int)
+        self.assertEqual('value', jsonutils.loads(response.body)['collection'])
+
+    def test_resource_extension_for_delete_custom_collection_method(self):
+        controller = self.ResourceExtensionController()
+        collections = {'custom_collection_method': "DELETE"}
+        res_ext = extensions.ResourceExtension('tweedles', controller,
+                                               collection_methods=collections)
+        test_app = _setup_extensions_test_app(SimpleExtensionManager(res_ext))
+
+        response = test_app.delete("/tweedles")
+
+        self.assertEqual(200, response.status_int)
+        self.assertEqual('value', jsonutils.loads(response.body)['collection'])
+
+    def test_resource_ext_for_formatted_req_on_custom_collection_method(self):
+        controller = self.ResourceExtensionController()
+        collections = {'custom_collection_method': "GET"}
+        res_ext = extensions.ResourceExtension('tweedles', controller,
+                                               collection_methods=collections)
+        test_app = _setup_extensions_test_app(SimpleExtensionManager(res_ext))
+
+        response = test_app.get("/tweedles.json")
+
+        self.assertEqual(200, response.status_int)
+        self.assertEqual("value", jsonutils.loads(response.body)['collection'])
+
+    def test_resource_ext_for_nested_resource_custom_collection_method(self):
+        controller = self.ResourceExtensionController()
+        collections = {'custom_collection_method': "GET"}
+        parent = {'collection_name': 'beetles', 'member_name': 'beetle'}
+        res_ext = extensions.ResourceExtension('tweedles', controller,
+                                               collection_methods=collections,
+                                               parent=parent)
+        test_app = _setup_extensions_test_app(SimpleExtensionManager(res_ext))
+
+        response = test_app.get("/beetles/beetle_id/tweedles")
+
+        self.assertEqual(200, response.status_int)
+        self.assertEqual("value", jsonutils.loads(response.body)['collection'])
+
     def test_resource_extension_with_custom_member_action_and_attr_map(self):
         controller = self.ResourceExtensionController()
         member = {'custom_member_action': "GET"}
@@ -500,6 +579,29 @@ class RequestExtensionTest(base.BaseTestCase):
 
 class ExtensionManagerTest(base.BaseTestCase):
 
+    def test_optional_extensions_no_error(self):
+        ext_mgr = extensions.ExtensionManager('')
+        attr_map = {}
+        ext_mgr.add_extension(ext_stubs.StubExtension('foo_alias',
+                                                      optional=['cats']))
+        ext_mgr.extend_resources("2.0", attr_map)
+        self.assertIn('foo_alias', ext_mgr.extensions)
+
+    def test_missing_required_extensions_raise_error(self):
+        ext_mgr = extensions.ExtensionManager('')
+        attr_map = {}
+        ext_mgr.add_extension(ext_stubs.StubExtensionWithReqs('foo_alias'))
+        self.assertRaises(exceptions.ExtensionsNotFound,
+                          ext_mgr.extend_resources, "2.0", attr_map)
+
+    def test_missing_required_extensions_gracefully_error(self):
+        ext_mgr = extensions.ExtensionManager('')
+        attr_map = {}
+        default_ext = list(constants.DEFAULT_SERVICE_PLUGINS.values())[0]
+        ext_mgr.add_extension(ext_stubs.StubExtensionWithReqs(default_ext))
+        ext_mgr.extend_resources("2.0", attr_map)
+        self.assertIn(default_ext, ext_mgr.extensions)
+
     def test_invalid_extensions_are_not_registered(self):
 
         class InvalidExtension(object):
