@@ -15,6 +15,7 @@
 
 import copy
 import os
+import re
 import sys
 import textwrap
 
@@ -27,6 +28,7 @@ import mock
 from oslo_utils import fileutils
 import pkg_resources
 import sqlalchemy as sa
+from testtools import matchers
 
 from neutron.db import migration
 from neutron.db.migration import autogen
@@ -728,8 +730,13 @@ class TestCli(base.BaseTestCase):
         self.assertTrue(expand.downgrade_ops.is_empty())
         self.assertTrue(contract.downgrade_ops.is_empty())
 
-        self.assertEqual(
-            textwrap.dedent("""\
+        def _get_regex(s):
+            s = textwrap.dedent(s)
+            s = re.escape(s)
+            # alembic 0.8.9 added additional leading '# ' before comments
+            return s.replace('\\#\\#\\#\\ ', '(# )?### ')
+
+        expected_regex = ("""\
             ### commands auto generated by Alembic - please adjust! ###
                 op.create_table('organization',
                 sa.Column('id', sa.Integer(), nullable=False),
@@ -740,17 +747,19 @@ class TestCli(base.BaseTestCase):
                 """sa.Column('organization_id', sa.Integer(), nullable=True))
                 op.create_foreign_key('org_fk', 'user', """
                 """'organization', ['organization_id'], ['id'])
-                ### end Alembic commands ###"""),
-            alembic_ag_api.render_python_code(expand.upgrade_ops)
-        )
-        self.assertEqual(
-            textwrap.dedent("""\
+                ### end Alembic commands ###""")
+        self.assertThat(
+            alembic_ag_api.render_python_code(expand.upgrade_ops),
+            matchers.MatchesRegex(_get_regex(expected_regex)))
+
+        expected_regex = ("""\
             ### commands auto generated by Alembic - please adjust! ###
                 op.drop_constraint('user', 'uq_user_org', type_=None)
                 op.drop_column('user', 'organization_name')
-                ### end Alembic commands ###"""),
-            alembic_ag_api.render_python_code(contract.upgrade_ops)
-        )
+                ### end Alembic commands ###""")
+        self.assertThat(
+            alembic_ag_api.render_python_code(contract.upgrade_ops),
+            matchers.MatchesRegex(_get_regex(expected_regex)))
 
     @mock.patch.object(cli, '_get_branch_points', return_value=[])
     @mock.patch.object(cli.CONF, 'split_branches',
