@@ -43,8 +43,8 @@ class TestIpamSubnetRequests(IpamSubnetRequestTestCase):
                                   self.subnet_id)
         self.assertEqual(self.tenant_id, pool.tenant_id)
         self.assertEqual(self.subnet_id, pool.subnet_id)
-        self.assertEqual(None, pool.gateway_ip)
-        self.assertEqual(None, pool.allocation_pools)
+        self.assertIsNone(pool.gateway_ip)
+        self.assertIsNone(pool.allocation_pools)
 
     def test_subnet_request_gateway(self):
         request = ipam_req.SubnetRequest(self.tenant_id,
@@ -134,7 +134,7 @@ class TestIpamAnySubnetRequest(IpamSubnetRequestTestCase):
 
     def test_subnet_request_bad_gateway(self):
         cfg.CONF.set_override('force_gateway_on_subnet', True)
-        self.assertRaises(ValueError,
+        self.assertRaises(ipam_exc.IpamValueInvalid,
                           ipam_req.AnySubnetRequest,
                           self.tenant_id,
                           self.subnet_id,
@@ -153,7 +153,7 @@ class TestIpamAnySubnetRequest(IpamSubnetRequestTestCase):
 
     def test_subnet_request_allocation_pool_wrong_version(self):
         pools = [netaddr.IPRange('0.0.0.4', '0.0.0.5')]
-        self.assertRaises(ValueError,
+        self.assertRaises(ipam_exc.IpamValueInvalid,
                           ipam_req.AnySubnetRequest,
                           self.tenant_id,
                           self.subnet_id,
@@ -163,7 +163,7 @@ class TestIpamAnySubnetRequest(IpamSubnetRequestTestCase):
 
     def test_subnet_request_allocation_pool_not_in_net(self):
         pools = [netaddr.IPRange('0.0.0.64', '0.0.0.128')]
-        self.assertRaises(ValueError,
+        self.assertRaises(ipam_exc.IpamValueInvalid,
                           ipam_req.AnySubnetRequest,
                           self.tenant_id,
                           self.subnet_id,
@@ -185,7 +185,7 @@ class TestIpamSpecificSubnetRequest(IpamSubnetRequestTestCase):
 
     def test_subnet_request_bad_gateway(self):
         cfg.CONF.set_override('force_gateway_on_subnet', True)
-        self.assertRaises(ValueError,
+        self.assertRaises(ipam_exc.IpamValueInvalid,
                           ipam_req.SpecificSubnetRequest,
                           self.tenant_id,
                           self.subnet_id,
@@ -383,8 +383,8 @@ class TestSubnetRequestFactory(IpamSubnetRequestTestCase):
                               ipam_req.SpecificSubnetRequest)
         self.assertEqual(self.tenant_id, request.tenant_id)
         self.assertEqual(self.subnet_id, request.subnet_id)
-        self.assertEqual(None, request.gateway_ip)
-        self.assertEqual(None, request.allocation_pools)
+        self.assertIsNone(request.gateway_ip)
+        self.assertIsNone(request.allocation_pools)
 
 
 class TestGetRequestFactory(base.BaseTestCase):
