@@ -46,8 +46,8 @@ class TestSriovAgentConfig(base.BaseTestCase):
 
     DEVICE_MAPPING_WITH_SPACES_LIST = ['physnet7 : p7p1',
                                        'physnet3 : p3p1 ']
-    DEVICE_MAPPING = {'physnet7': 'p7p1',
-                      'physnet3': 'p3p1'}
+    DEVICE_MAPPING = {'physnet7': ['p7p1'],
+                      'physnet3': ['p3p1']}
 
     def test_defaults(self):
         self.assertEqual(config.DEFAULT_DEVICE_MAPPINGS,
@@ -62,7 +62,7 @@ class TestSriovAgentConfig(base.BaseTestCase):
                               self.DEVICE_MAPPING_LIST,
                               'SRIOV_NIC')
         device_mappings = n_utils.parse_mappings(
-            cfg.CONF.SRIOV_NIC.physical_device_mappings)
+            cfg.CONF.SRIOV_NIC.physical_device_mappings, unique_keys=False)
         self.assertEqual(self.DEVICE_MAPPING, device_mappings)
 
     def test_device_mappings_with_error(self):
@@ -70,14 +70,15 @@ class TestSriovAgentConfig(base.BaseTestCase):
                               self.DEVICE_MAPPING_WITH_ERROR_LIST,
                               'SRIOV_NIC')
         self.assertRaises(ValueError, n_utils.parse_mappings,
-                          cfg.CONF.SRIOV_NIC.physical_device_mappings)
+                          cfg.CONF.SRIOV_NIC.physical_device_mappings,
+                          unique_keys=False)
 
     def test_device_mappings_with_spaces(self):
         cfg.CONF.set_override('physical_device_mappings',
                               self.DEVICE_MAPPING_WITH_SPACES_LIST,
                               'SRIOV_NIC')
         device_mappings = n_utils.parse_mappings(
-            cfg.CONF.SRIOV_NIC.physical_device_mappings)
+            cfg.CONF.SRIOV_NIC.physical_device_mappings, unique_keys=False)
         self.assertEqual(self.DEVICE_MAPPING, device_mappings)
 
     def test_exclude_devices(self):
