@@ -96,7 +96,10 @@ class TestESwitchManagerApi(base.BaseTestCase):
     def test_get_assigned_devices_info(self):
         with mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent."
                         "eswitch_manager.EmbSwitch.get_assigned_devices_info",
-                        return_value=[(self.ASSIGNED_MAC, self.PCI_SLOT)]):
+                        return_value=[(self.ASSIGNED_MAC, self.PCI_SLOT)]),\
+                mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent."
+                           "pci_lib.PciDeviceIPWrapper.link_show",
+                           return_value=''):
             result = self.eswitch_mgr.get_assigned_devices_info()
             self.assertIn(self.ASSIGNED_MAC, list(result)[0])
             self.assertIn(self.PCI_SLOT, list(result)[0])
@@ -232,6 +235,9 @@ class TestESwitchManagerApi(base.BaseTestCase):
     @mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent.pci_lib."
                 "PciDeviceIPWrapper.get_assigned_macs",
                 return_value={})
+    @mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent."
+                "pci_lib.PciDeviceIPWrapper.link_show",
+                return_value='')
     @mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent."
                 "eswitch_manager.EmbSwitch.set_device_max_rate")
     def test_clear_max_rate_existing_pci_slot(self, max_rate_mock, *args):
@@ -241,6 +247,9 @@ class TestESwitchManagerApi(base.BaseTestCase):
     @mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent.pci_lib."
                 "PciDeviceIPWrapper.get_assigned_macs",
                 return_value={0: ASSIGNED_MAC})
+    @mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent."
+                "pci_lib.PciDeviceIPWrapper.link_show",
+                return_value='')
     @mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent."
                 "eswitch_manager.EmbSwitch.set_device_max_rate")
     def test_clear_max_rate_exist_and_assigned_pci(
@@ -291,7 +300,10 @@ class TestEmbSwitch(base.BaseTestCase):
                         return_value={0: self.ASSIGNED_MAC}),\
                 mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent."
                            "eswitch_manager.PciOsWrapper.is_assigned_vf",
-                           return_value=True):
+                           return_value=True), \
+                mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent."
+                           "pci_lib.PciDeviceIPWrapper.link_show",
+                           return_value=''):
             result = emb_switch.get_assigned_devices_info()
             self.assertIn(self.ASSIGNED_MAC, list(result)[0])
             self.assertIn(self.PCI_SLOT, list(result)[0])
@@ -306,7 +318,10 @@ class TestEmbSwitch(base.BaseTestCase):
                         return_value=self.VF_TO_MAC_MAPPING),\
                 mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent."
                            "eswitch_manager.PciOsWrapper.is_assigned_vf",
-                           return_value=True):
+                           return_value=True),\
+                mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent."
+                           "pci_lib.PciDeviceIPWrapper.link_show",
+                           return_value=''):
             devices_info = emb_switch.get_assigned_devices_info()
             for device_info in devices_info:
                 mac = device_info[0]
@@ -317,7 +332,10 @@ class TestEmbSwitch(base.BaseTestCase):
     def test_get_assigned_devices_empty(self):
         with mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent."
                         "eswitch_manager.PciOsWrapper.is_assigned_vf",
-                        return_value=False):
+                        return_value=False), \
+             mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent."
+                        "pci_lib.PciDeviceIPWrapper.link_show",
+                        return_value=''):
             result = self.emb_switch.get_assigned_devices_info()
             self.assertFalse(result)
 
@@ -414,7 +432,10 @@ class TestEmbSwitch(base.BaseTestCase):
                         return_value={0: self.ASSIGNED_MAC}),\
                 mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent."
                            "eswitch_manager.PciOsWrapper.is_assigned_vf",
-                           return_value=True):
+                           return_value=True),\
+                mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent."
+                           "pci_lib.PciDeviceIPWrapper.link_show",
+                           return_value=''):
             result = self.emb_switch.get_pci_device(self.PCI_SLOT)
             self.assertEqual(self.ASSIGNED_MAC, result)
 
@@ -492,21 +513,21 @@ class TestPciOsWrapper(base.BaseTestCase):
     @mock.patch("os.listdir", side_effect=OSError())
     def test_is_assigned_vf_true(self, *args):
         self.assertTrue(esm.PciOsWrapper.is_assigned_vf(
-            self.DEV_NAME, self.VF_INDEX))
+            self.DEV_NAME, self.VF_INDEX, ''))
 
     @mock.patch("os.listdir", return_value=[DEV_NAME, "eth1"])
     @mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent.pci_lib."
                 "PciDeviceIPWrapper.is_macvtap_assigned", return_value=False)
     def test_is_assigned_vf_false(self, *args):
         self.assertFalse(esm.PciOsWrapper.is_assigned_vf(
-            self.DEV_NAME, self.VF_INDEX))
+            self.DEV_NAME, self.VF_INDEX, ''))
 
     @mock.patch("os.listdir", return_value=["eth0", "eth1"])
     @mock.patch("neutron.plugins.ml2.drivers.mech_sriov.agent.pci_lib."
                 "PciDeviceIPWrapper.is_macvtap_assigned", return_value=True)
     def test_is_assigned_vf_macvtap(
         self, mock_is_macvtap_assigned, *args):
-        esm.PciOsWrapper.is_assigned_vf(self.DEV_NAME, self.VF_INDEX)
+        esm.PciOsWrapper.is_assigned_vf(self.DEV_NAME, self.VF_INDEX, '')
         mock_is_macvtap_assigned.called_with(self.VF_INDEX, "eth0")
 
     @mock.patch("os.listdir", side_effect=OSError())
@@ -514,5 +535,5 @@ class TestPciOsWrapper(base.BaseTestCase):
                 "PciDeviceIPWrapper.is_macvtap_assigned")
     def test_is_assigned_vf_macvtap_failure(
         self, mock_is_macvtap_assigned, *args):
-        esm.PciOsWrapper.is_assigned_vf(self.DEV_NAME, self.VF_INDEX)
+        esm.PciOsWrapper.is_assigned_vf(self.DEV_NAME, self.VF_INDEX, '')
         self.assertFalse(mock_is_macvtap_assigned.called)
