@@ -389,6 +389,10 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
                             src_field='arp_spa'),
                         ofpp.OFPActionSetField(arp_sha=mac),
                         ofpp.OFPActionSetField(arp_spa=ip),
+                        ofpp.NXActionRegMove(src_field='eth_src',
+                                             dst_field='eth_dst',
+                                             n_bits=48),
+                        ofpp.OFPActionSetField(eth_src_nxm=mac),
                         ofpp.OFPActionOutput(ofp.OFPP_IN_PORT, 0),
                     ]),
                 ],
